Package: multiDEGGs
Title: Multi-Omic Differentially Expressed Gene-Gene Pairs
Version: 1.2.1
Authors@R: 
    c(person(given = "Elisabetta",
          family = "Sciacca",
          role = c("aut", "cre", "cph"),
          email = "e.sciacca@qmul.ac.uk",
          comment = c(ORCID = "0000-0001-7525-1558")),
      person(given = "Myles",family = "Lewis",
          role = c("ctb"),
          email = "myles.lewis@qmul.ac.uk",
          comment = c(ORCID = "0000-0001-9365-5345")))
Maintainer: Elisabetta Sciacca <e.sciacca@qmul.ac.uk>
Description: Performs multi-omic differential network 
      analysis by revealing differential interactions between molecular entities
      (genes, proteins, transcription factors, or other biomolecules) across the
      omic datasets provided. 
      For each omic dataset, a differential network is constructed where
      links represent statistically significant differential interactions between 
      entities. These networks are then integrated into a comprehensive visualization
      using distinct colors to distinguish interactions from different omic layers.
      This unified display allows interactive exploration of cross-omic
      patterns, such as differential interactions present at both transcript and
      protein levels. For each link, users can access differential statistical 
      significance metrics (p values or adjusted p values, calculated via robust or 
      traditional linear regression with interaction term) and differential regression
      plots.  
      The methods implemented in this package are described in Sciacca et al. (2023)
      <doi:10.1093/bioinformatics/btad192>.
License: GPL-3
Encoding: UTF-8
LazyData: true
LazyDataCompression: gzip
RoxygenNote: 7.3.3
Language: en-gb
URL: https://github.com/elisabettasciacca/multiDEGGs/,
        https://elisabettasciacca.github.io/multiDEGGs/
BugReports: https://github.com/elisabettasciacca/multiDEGGs/issues
Suggests: kernlab, nestedcv, pls, qvalue, randomForest, ranger,
        testthat (>= 3.0.0)
Imports: DT, grDevices, graphics, knitr, MASS, magrittr, methods,
        parallel, pbapply, pbmcapply, rmarkdown, sfsmisc, shiny,
        shinydashboard, stats, utils, visNetwork
Depends: R (>= 4.4.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-10 09:09:30 UTC; elisabetta
Author: Elisabetta Sciacca [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-7525-1558>),
  Myles Lewis [ctb] (ORCID: <https://orcid.org/0000-0001-9365-5345>)
Repository: CRAN
Date/Publication: 2026-04-10 09:40:02 UTC
Built: R 4.5.2; ; 2026-04-10 11:19:43 UTC; unix

Package: multiDEGGs
Title: Multi-Omic Differentially Expressed Gene-Gene Pairs
Version: 1.2.1
Authors@R: 
    c(person(given = "Elisabetta",
          family = "Sciacca",
          role = c("aut", "cre", "cph"),
          email = "e.sciacca@qmul.ac.uk",
          comment = c(ORCID = "0000-0001-7525-1558")),
      person(given = "Myles",family = "Lewis",
          role = c("ctb"),
          email = "myles.lewis@qmul.ac.uk",
          comment = c(ORCID = "0000-0001-9365-5345")))
Maintainer: Elisabetta Sciacca <e.sciacca@qmul.ac.uk>
Description: Performs multi-omic differential network 
      analysis by revealing differential interactions between molecular entities
      (genes, proteins, transcription factors, or other biomolecules) across the
      omic datasets provided. 
      For each omic dataset, a differential network is constructed where
      links represent statistically significant differential interactions between 
      entities. These networks are then integrated into a comprehensive visualization
      using distinct colors to distinguish interactions from different omic layers.
      This unified display allows interactive exploration of cross-omic
      patterns, such as differential interactions present at both transcript and
      protein levels. For each link, users can access differential statistical 
      significance metrics (p values or adjusted p values, calculated via robust or 
      traditional linear regression with interaction term) and differential regression
      plots.  
      The methods implemented in this package are described in Sciacca et al. (2023)
      <doi:10.1093/bioinformatics/btad192>.
License: GPL-3
Encoding: UTF-8
LazyData: true
LazyDataCompression: gzip
RoxygenNote: 7.3.3
Language: en-gb
URL: https://github.com/elisabettasciacca/multiDEGGs/,
        https://elisabettasciacca.github.io/multiDEGGs/
BugReports: https://github.com/elisabettasciacca/multiDEGGs/issues
Suggests: kernlab, nestedcv, pls, qvalue, randomForest, ranger,
        testthat (>= 3.0.0)
Imports: DT, grDevices, graphics, knitr, MASS, magrittr, methods,
        parallel, pbapply, pbmcapply, rmarkdown, sfsmisc, shiny,
        shinydashboard, stats, utils, visNetwork
Depends: R (>= 4.4.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-10 09:09:30 UTC; elisabetta
Author: Elisabetta Sciacca [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-7525-1558>),
  Myles Lewis [ctb] (ORCID: <https://orcid.org/0000-0001-9365-5345>)
Repository: CRAN
Date/Publication: 2026-04-10 09:40:02 UTC
Built: R 4.6.0; ; 2026-04-29 22:28:14 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(predict,multiDEGGs_filter)
S3method(predict,multiDEGGs_filter_combined)
export(.predict_multiDEGGs)
export(View_diffNetworks)
export(get_diffNetworks)
export(get_multiOmics_diffNetworks)
export(get_sig_deggs)
export(multiDEGGs_combined_filter)
export(multiDEGGs_filter)
export(plot_regressions)
import(knitr)
import(rmarkdown)
importFrom(MASS,rlm)
importFrom(grDevices,adjustcolor)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,boxplot)
importFrom(graphics,legend)
importFrom(graphics,mtext)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(sfsmisc,f.robftest)

# multiDEGGs
## Differentially Expressed Gene-Gene pairs in multi omic data
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![License: GPL v3](https://img.shields.io/badge/License-GPL%20v3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.html)
[![CRAN status](https://www.r-pkg.org/badges/version/multiDEGGs)](https://CRAN.R-project.org/package=multiDEGGs)
[![R-CMD-check](https://github.com/elisabettasciacca/multiDEGGs/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/elisabettasciacca/multiDEGGs/actions/workflows/R-CMD-check.yaml)  
[![downloads](https://cranlogs.r-pkg.org/badges/grand-total/multiDEGGs)](https://cran.r-project.org/package=multiDEGGs)
[![last commit](https://img.shields.io/github/last-commit/elisabettasciacca/multiDEGGs.svg)](https://github.com/elisabettasciacca/multiDEGGs/commits/master)

# multiDEGGs <a href="https://elisabettasciacca.github.io/multiDEGGs/"><img src="man/figures/logo.png" align="right" height="137" alt="multiDEGGs website" /></a>

The multiDEGGs package test for differential gene-gene correlations across different groups of samples in multi omic data.  
Specific gene-gene interactions can be explored and gene-gene pair regression plots can be interactively shown.   

### Installation 
Install from CRAN:    
`install.packages("multiDEGGs")`
  
Install from Github:  
`devtools::install_github("elisabettasciacca/multiDEGGs")`

### Example  
Load package and sample data   
```r
library(multiDEGGs)  
data("synthetic_metadata")  
data("synthetic_rnaseqData")  
data("synthetic_proteomicData")
data("synthetic_OlinkData")   
```
  
Generate differential networks:   
```r
assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                       "Proteomics" = synthetic_proteomicData,
                       "Olink" = synthetic_OlinkData)

deggs_object <- get_diffNetworks(assayData = assayData_list,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 regression_method = "lm",
                                 padj_method = "bonferroni",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 2)
```
  
Visualise interactively (will open a shiny interface)   
```r
View_diffNetworks(deggs_object)
```  
  
Get a table listing all the significant interactions found in each category  
```r
get_multiOmics_diffNetworks(deggs_object, sig_threshold = 0.05)
```
   
Plot differential regression fits for a single interaction  
`plot_regressions(deggs_object,
                 assayDataName = "RNAseq",
                 gene_A = "MTOR", 
                 gene_B = "AKT2",
                 legend_position = "bottomright")`
                  
## Citation
To cite package ‘multiDEGGs’ in publications use:

  Sciacca E, Wang S, Pitzalis C, Lewis M (2026). “multiDEGGs: Single or Multiomic
  Differential Network Analysis for Biomarker Discovery and Feature Engineering for
  Predictive Modeling.” _Computational and Structural Biotechnology Journal_, *35*(1),
  0001. doi:10.34133/csbj.0001 <https://doi.org/10.34133/csbj.0001>.

A BibTeX entry for LaTeX users is

  @Article{,
    title = {multiDEGGs: Single or Multiomic Differential Network Analysis for Biomarker Discovery and Feature Engineering for Predictive Modeling},
    author = {Elisabetta Sciacca and Susan S. Wang and Costantino Pitzalis and Myles J. Lewis},
    journal = {Computational and Structural Biotechnology Journal},
    year = {2026},
    volume = {35},
    number = {1},
    pages = {0001},
    doi = {10.34133/csbj.0001},
  }
  

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval=FALSE---------------------------------------------------------------
# library(multiDEGGs)
# library(nestedcv)
# data("synthetic_metadata")
# data("synthetic_rnaseqData")
# 
# # Regularized linear model with interaction pairs only
# fit.glmnet <- nestcv.glmnet(
#   y = as.numeric(synthetic_metadata$response),
#   x = t(synthetic_rnaseqData),
#   modifyX = "multiDEGGs_filter",
#   modifyX_options = list(
#     keep_single_genes = FALSE,
#     nfilter = 20
#   ),
#   modifyX_useY = TRUE,
#   n_outer_folds = 5,
#   n_inner_folds = 6,
#   verbose = FALSE
# )
# 
# summary(fit.glmnet)

## ----echo=FALSE---------------------------------------------------------------
cat("Nested cross-validation with glmnet 
No filter 
Modifier:  multiDEGGs_filter 
Outer loop:  5-fold CV 
Inner loop:  6-fold CV 
100 observations, 14 predictors 
 
 
Final parameters: 
 lambda    alpha   
0.05894  0.10000   

Final coefficients: 
 (Intercept) TNF:TNFRSF1A    AKT2:MTOR   IL1B:IL1R2    FASLG:FAS TGFB3:TGFBR1  
    1.823874    -0.193020    -0.119887     0.052089    -0.035947    -0.033527  
MAP2K2:MAPK3  FANCD2:FAN1  
   -0.021308    -0.008862  
 
Result: 
       RMSE     R.squared   Pearson.r^2           MAE    
    0.47302       0.08148       0.09173       0.44153   " )

## ----eval=FALSE---------------------------------------------------------------
# # Random forest model including both pairs and individual genes
# fit.rf <- nestcv.train(
#   y = synthetic_metadata$response,
#   x = t(synthetic_rnaseqData),
#   method = "rf",
#   modifyX = "multiDEGGs_filter",
#   modifyX_options = list(
#     keep_single_genes = TRUE,
#     nfilter = 30
#   ),
#   modifyX_useY = TRUE,
#   n_outer_folds = 5,
#   n_inner_folds = 6,
#   verbose = FALSE
# )
# 
# fit.rf$summary
# 
# # Plot ROC on outer folds
# plot(fit.rf$roc)

## ----echo=FALSE---------------------------------------------------------------
cat("               Reference
Predicted       Non_responder Responder
  Non_responder            57         2
  Responder                 1        40

              AUC            Accuracy   Balanced accuracy   
           0.9979              0.9700              0.9676   " )

## ----eval=FALSE---------------------------------------------------------------
# # Dynamic selection with t-test for single genes
# fit.dynamic <- nestcv.glmnet(
#   y = as.numeric(synthetic_metadata$response),
#   x = t(synthetic_rnaseqData),
#   modifyX = "multiDEGGs_combined_filter",
#   modifyX_options = list(
#     filter_method = "ttest",
#     nfilter = 20,
#     dynamic_nfilter = TRUE,
#     keep_single_genes = FALSE
#   ),
#   modifyX_useY = TRUE,
#   n_outer_folds = 5,
#   n_inner_folds = 6,
#   verbose = FALSE
# )

## ----eval=FALSE---------------------------------------------------------------
# # Balanced selection with Wilcoxon-test importance
# fit.balanced <- nestcv.train(
#   y = synthetic_metadata$response,
#   x = t(synthetic_rnaseqData),
#   method = "rf",
#   modifyX = "multiDEGGs_combined_filter",
#   modifyX_options = list(
#     filter_method = "wilcoxon",
#     nfilter = 40,
#     dynamic_nfilter = FALSE
#   ),
#   modifyX_useY = TRUE,
#   n_outer_folds = 5,
#   n_inner_folds = 6,
#   verbose = FALSE
# )

## -----------------------------------------------------------------------------
citation("multiDEGGs")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----load_data----------------------------------------------------------------
library(multiDEGGs)
data("synthetic_metadata")
data("synthetic_rnaseqData")
data("synthetic_proteomicData")
data("synthetic_OlinkData")

## -----------------------------------------------------------------------------
assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                       "Proteomics" = synthetic_proteomicData,
                       "Olink" = synthetic_OlinkData)

deggs_object <- get_diffNetworks(assayData = assayData_list,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 regression_method = "lm",
                                 padj_method = "bonferroni",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 2)

## ----eval=FALSE---------------------------------------------------------------
# View_diffNetworks(deggs_object)

## ----warning=FALSE------------------------------------------------------------
get_multiOmics_diffNetworks(deggs_object, sig_threshold = 0.05)

## -----------------------------------------------------------------------------
deggs_object_oneOmic <- get_diffNetworks(assayData = synthetic_rnaseqData,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 regression_method = "lm",
                                 padj_method = "bonferroni",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 2)

get_sig_deggs(deggs_object_oneOmic, sig_threshold = 0.05)

## ----fig.width = 4.5, fig.height = 4, eval=FALSE------------------------------
# plot_regressions(deggs_object,
#                  assayDataName = "RNAseq",
#                  gene_A = "MTOR",
#                  gene_B = "AKT2",
#                  legend_position = "bottomright")

## -----------------------------------------------------------------------------
citation("multiDEGGs")

#' Synthetic clinical data
#'
#' A dataset containing sample clinical data for 100 patients with 40% 
#' response rate
#' 
#' @format A data frame with 100 rows and 4 columns (IDs are in rownames):
#' \describe{
#'   \item{patient_id}{IDs matching the IDs used in the colnames of the assay
#'   data matrix/matrices.}
#'   \item{age}{A column to simulate age of patients. Not used.}
#'   \item{gender}{A column to simulate gender of patients. Not used.}
#'   \item{response}{The response outcome, to be used for differential analysis}
#' }
#' @usage NULL
"synthetic_metadata"

#' Synthetic RNA-seq count data
#'
#' Synthetic RNA-seq data after log2 normalisation
#'
#' @format A data frame with xx rows (genes) xx columns (patients IDs, matching 
#' the metadata rownames).
#' @usage NULL
"synthetic_rnaseqData"

#' Synthetic RNA-seq count data
#'
#' Synthetic RNA-seq data after log2 normalisation
#'
#' @format A data frame with xx rows (proteins) xx columns (patients IDs).
#' @usage NULL
"synthetic_proteomicData"

#' Synthetic RNA-seq count data
#'
#' Synthetic RNA-seq data after log2 normalisation
#'
#' @format A data frame with xx rows (proteins) xx columns (patients IDs).
#' @usage NULL
"synthetic_OlinkData"

#' Internal function for colors 
#'
#' This function return a color palette with the number of colors specified by n
#'
#' @param n number of colors needed
#' @return a vector with colors
my_palette <- function(n) {
  palette <- c("#3B4992", "#DF8F44", "#B24745", "#1B5E20", "#6B452B", "#8F7700",
               "#808180", "#91D1C2","#FABFD2")
  return(palette[seq_len(n)])
}

#' Plot differential regressions for a link
#' 
#' Plot differential regressions for any target-target pair in an omic dataset
#'
#' @param deggs_object an object of class `deggs` generated by
#' `get_diffNetworks`
#' @param assayDataName name of the assayData of interest. If an unnamed list of 
#' data was given to `get_diffNetworks`, the assayDataName here will be the 
#' number indicating the position of the data in the assayDataList provided
#' before (i.e. if the user wants to plot a differential interaction observed in 
#' the transcriptomic data, which was second in the list, then assayDataName 
#' must be 2, if only one data table was provided assayDataName must be 1). 
#' Default 1.
#' @param gene_A character. Name of the first target (gene, protein, metabolite,
#' etc.)
#' @param gene_B character. Name of the second target (gene, protein, metabolite,
#' etc.)
#' @param title plot title. If NULL (default), the name of the assayData will be 
#' used. Use empty character "" for no title. 
#' @param legend_position position of the legend in the plot. It can be
#' specified by keyword or in any parameter accepted by `xy.coords` (defalut
#' "topright")
#' @importFrom methods is
#' @importFrom grDevices adjustcolor
#' @importFrom graphics abline axis boxplot legend mtext points polygon text
#' @return base graphics plot showing differential regressions across 
#' categories. The p value of the interaction term of
#' gene A ~ gene B \* category is reported on top.
#' @examples
#' data("synthetic_metadata")
#' data("synthetic_rnaseqData")
#' data("synthetic_proteomicData")
#' data("synthetic_OlinkData")
#' assayData_list <- list("RNAseq" = synthetic_rnaseqData,
#'                        "Proteomics" = synthetic_proteomicData,
#'                        "Olink" = synthetic_OlinkData)
#' deggs_object <- get_diffNetworks(assayData = assayData_list,
#'                                  metadata = synthetic_metadata,
#'                                  category_variable = "response",
#'                                  regression_method = "lm",
#'                                  padj_method = "bonferroni",
#'                                  verbose = FALSE,
#'                                  show_progressBar = FALSE,
#'                                  cores = 1)
#' plot_regressions(deggs_object,
#'                  assayDataName = "RNAseq",
#'                  gene_A = "MTOR", 
#'                  gene_B = "AKT2",
#'                  legend_position = "bottomright")
#' @export
plot_regressions <- function(deggs_object,
                             assayDataName = 1,
                             gene_A,
                             gene_B,
                             title = NULL, 
                             legend_position = "topright") {
  
  if (!is(deggs_object, "deggs")) {
    stop("deggs_object must be of class deggs")
  }
  
  if (!(is.character(gene_A) && is.character(gene_B))) {
    stop("Both GeneA and GeneB must be character.")
  }
  
  sig_var <- ifelse(deggs_object[["padj_method"]] == "none", "p.value", "p.adj")
  metadata <- deggs_object[["metadata"]]
  assayData <- deggs_object[["assayData"]][[assayDataName]]
  categories <- deggs_object[["category_subset"]]
  regression_method <- deggs_object[["regression_method"]]
  
  if (!gene_A %in% rownames(assayData)) (
    stop("gene_A is not in rownames(", assayDataName, ")")
  )
  
  if (!gene_B %in% rownames(assayData)) (
    stop("gene_B is not in rownames(", assayDataName, ")")
  )
  
  metadata <- metadata[colnames(assayData)]
  
  if(length(unique(metadata)) == 1) (
    stop("All sample IDs in ", assayDataName, " belong to one category. 
         No differential analysis is possible.")
  )
  
  if (is.null(categories)) {
    categories <- levels(metadata)
  }
  
  category_length <- length(categories)
  if(is.null(title)) (
    title <- ifelse(is.numeric(assayDataName), 
                    names(deggs_object[["diffNetworks"]])[assayDataName],
                    assayDataName)
  )
  
  # prepare data frame
  # using both t() and as.vector() to be compatible with both matrices and dfs
  df <- data.frame(as.vector(t(assayData[gene_A, ])),
                   as.vector(t(assayData[gene_B, ])),
                   metadata,
                   check.names = FALSE)
  colnames(df) <- c(gene_A, gene_B, "category")
  
  # compute gene-gene regression
  if (category_length == 2) {
    if (regression_method == "lm") {
      lmfit <- stats::lm(df[, 2] ~ df[, 1] * df[, 3])
      # i.e.: gene_A ~ gene_B * category
      p_interaction <- stats::coef(summary(lmfit))[4, 4]
      fit <- lapply(categories, function(i) {
        x <- df[df[, "category"] == i, 1]
        y <- df[df[, "category"] == i, 2]
        if (length(x) > 0 || length(y) > 0) return(stats::lm(y ~ x))
      })
    }
    if (regression_method == "rlm") {
      robustfit <- MASS::rlm(df[, 2] ~ df[, 1] * df[, 3])
      # i.e.: gene_A ~ gene_B * category
      p_interaction <- sfsmisc::f.robftest(robustfit, var = 3)$p.value
      fit <- lapply(categories, function(i) {
        x <- df[df[, "category"] == i, 1]
        y <- df[df[, "category"] == i, 2]
        if (length(x) > 0 || length(y) > 0) return(MASS::rlm(y ~ x))
      })
    }
  }
  if (category_length >= 3) {
    # one-way ANOVA
    # i.e.: gene_A ~ gene_B * category
    res_aov <- stats::aov(df[, 2] ~ df[, 1] * df[, 3], data = df)
    p_interaction <- summary(res_aov)[[1]][["Pr(>F)"]][3]
    fit <- lapply(categories, function(i) {
      x <- df[df[, "category"] == i, 1]
      y <- df[df[, "category"] == i, 2]
      if (regression_method == "lm") {
        if (length(x) > 0 || length(y) > 0) return(stats::lm(y ~ x));
      }
      if (regression_method == "rlm") {
        if (length(x) > 0 || length(y) > 0) return(MASS::rlm(y ~ x))
      }
    })
  }
  
  # Plot
  prefix <- ifelse(deggs_object[["padj_method"]] == "none", "P", "Padj")
  col <- my_palette(n = category_length)
  x_adj <- (max(df[, 1], na.rm = TRUE) - min(df[, 1], na.rm = TRUE)) * 0.05
  new_x <- seq(min(df[, 1], na.rm = TRUE) - x_adj,
               max(df[, 1], na.rm = TRUE) + x_adj,
               length.out = 100
  )
  
  # prediction of the fitted model
  preds <- lapply(fit, 
                  function(i) stats::predict(i,
                                             newdata = data.frame(x = new_x),
                                             interval = 'confidence')
                  )
  
  # we have the interaction p value for a single pair,
  # adjusted p values can be found in the deggs_object if padj_method was NOT
  # set to none
  if (deggs_object[["padj_method"]] != "none") {
    all_interactions <- do.call(rbind, 
                                deggs_object[["diffNetworks"]][[assayDataName]][categories])
    pair_index <- which(all_interactions$from == gene_A & 
                          all_interactions$to == gene_B)
    # if you don't find gene_A-gene_B, try gene_B-gene_A:
    if(length(pair_index) == 0)(
      pair_index <- which(all_interactions$from == gene_B & 
                            all_interactions$to == gene_A)
    )
    sig_interaction <- all_interactions[pair_index, sig_var]
  } else {
    sig_interaction <- p_interaction
  }
  
  plot(df[, 1], df[, 2],
       type = 'n', bty = 'l', las = 1, cex.axis = 1.1,
       font.main = 1, cex.lab = 1.3, xlab = colnames(df)[1],
       ylab = colnames(df)[2],
       main = title
  )
  
  for (i in seq_along(categories)) {
    # plot confidence intervals
    polygon(c(rev(new_x), new_x), c(rev(preds[[i]][, 3]), preds[[i]][, 2]),
            col = adjustcolor(col[i], alpha.f = 0.15), border = NA
    )
    
    # plot regression lines
    abline(fit[[i]], col = col[i], lwd = 1.5)
    
    # plot dots 
    cols <- col[df[df[, "category"] == categories[i], 3]]
    pch <- c(16:(16 + category_length - 1))[df[df[, "category"] ==
                                                 categories[i], 3]]
    row <- points(df[df[, "category"] == categories[i], 1], # x coordinates from gene_A 
                  df[df[, "category"] == categories[i], 2], # y coordinates from gene_B 
                  cex = 1.5,
                  pch = pch, col = adjustcolor(cols, alpha.f = 0.7)
    )
  }
  mtext(
    bquote(paste(
      .(prefix)["interaction"] * "=",
      .(format(sig_interaction, digits = 2))
    )),
    cex = 1.2, side = 3, adj = 0.04
  )
  legend(
    x = legend_position, legend = categories,
    col = col, lty = 1,
    bty = "o", box.lty = 0,
    cex = 0.8
  )
}

#' Boxplots of single nodes (genes,proteins, etc.)
#'
#' This function is for internal use of `View_diffnetworks`
#'
#' @param gene gene name  (must be in `rownames(assayData)`)
#' @param assayDataName name of the assayData of interest. If an unnamed list of 
#' data was given to `get_diffNetworks`, the assayDataName here will be the 
#' number indicating the position of the data in the assayDataList provided
#' before (i.e. if the user wants to plot a differential interaction observed in 
#' the transcriptomic data, which was second in the list, then assayDataName 
#' must be 2, if only one data table was provided assayDataName must be 1). 
#' Default 1.
#' @param deggs_object  an object of class `deggs` generated by 
#' `get_diffNetworks`
#' @return the boxplot
node_boxplot <- function(gene,
                         assayDataName = 1,
                         deggs_object) {
  
  title <- ifelse(is.numeric(assayDataName), 
                  names(deggs_object[["diffNetworks"]])[assayDataName],
                  assayDataName)
  
  # if the selected node is not in present in the first assayData matrix 
  # show an empty plot with an informative message
  if (!gene %in% rownames(deggs_object[["assayData"]][[assayDataName]])) {
    plot(x = 0:10, y = 0:10, ann = FALSE,bty = "n",type = "n",
         xaxt = "n", yaxt = "n")
    text(x = 5, y = 9, labels = paste(gene, "is not present \n in", title), 
         font = 2, cex = 1.3)
    return()
  }
  metadata <- deggs_object[["metadata"]]
  x <- metadata[colnames(deggs_object[["assayData"]][[assayDataName]])]
  y <- as.numeric(deggs_object[["assayData"]][[assayDataName]][gene, ])
  col <- my_palette(n = nlevels(metadata))
  cols <- col[as.numeric(x)]
  
  boxplot(y ~ x,
          outline = FALSE, whisklty = 1, medlwd = 2, cex.axis = 1.1,
          col = NA, cex.lab = 1.3, ylab = gene, las = 2, boxwex = .5,
          ylim = range(y), xaxt = 'n', xlab = '',
          main = title)
  
  points(jitter(as.numeric(x), amount = 0.15), y,
         pch = 20, col = adjustcolor(cols, alpha.f = 0.7), cex = 2)
  
  xtick <- levels(x)
  axis(1, at = seq_along(xtick), labels = xtick, cex.axis = 1.2)
}


#' Interactive visualisation of differential networks
#'
#' Explore differential networks and interactively select regression 
#' and box plots
#'
#' @param deggs_object an object of class `deggs` generated by
#' `get_diffNetworks`
#' @param legend.arrow.width width of the arrow used in the network legend. 
#' Default is 0.35. As the number of assayData matrices increases this parameter
#' must be accordingly increased to avoid graphical errors in the legend. 
#' @param stepY_legend vertical space between legend arrows. It is used together 
#' with `legend.arrow.width` to adjust the legend space in case of graphical
#' errors. Default is 55. 
#' @import knitr
#' @import rmarkdown
#' @importFrom magrittr %>%
#' @return a shiny interface showing networks with selectable nodes and links
#' @examples 
#' data("synthetic_metadata")
#' data("synthetic_rnaseqData")
#' data("synthetic_proteomicData")
#' data("synthetic_OlinkData")
#' assayData_list <- list("RNAseq" = synthetic_rnaseqData,
#'                        "Proteomics" = synthetic_proteomicData,
#'                        "Olink" = synthetic_OlinkData)
#' deggs_object <- get_diffNetworks(assayData = assayData_list,
#'                                  metadata = synthetic_metadata,
#'                                  category_variable = "response",
#'                                  regression_method = "lm",
#'                                  verbose = FALSE,
#'                                  show_progressBar = FALSE,
#'                                  cores = 1)
#' # the below function runs a shiny app, so can't be run during R CMD check                                  
#' if(interactive()){
#' View_diffNetworks(deggs_object)  
#' }
#' @export
View_diffNetworks <- function(deggs_object,
                              legend.arrow.width = 0.35,
                              stepY_legend = 55) {
  
  if (!is(deggs_object, "deggs")) stop("deggs_object must be of class deggs")
  
  sig_var <- ifelse(deggs_object[["padj_method"]] == "none", "p.value", "p.adj")
  multiOmic <- ifelse(length(deggs_object[["assayData"]]) > 1, TRUE, FALSE)
  
  if (multiOmic) {
    category_networks <- get_multiOmics_diffNetworks(deggs_object)
  } else {
    category_networks <- deggs_object[["diffNetworks"]][[1]]
  }
  
  ################# server ########################
  server <- function(input, output, session) {
    empty_df <- data.frame("from" = NA, "to" = NA, "p.value" = 1, "p.adj" = 1, 
                           "layer" = NA)
    
    # check if the category_network has data 
    outVar <- shiny::reactive({
      if (!is.data.frame(category_networks[[input$category]])) return(empty_df)
      if (nrow(category_networks[[input$category]]) == 0) return(empty_df)
      return(category_networks[[input$category]])
    })
    
    nodes_selection <- shiny::reactiveValues(current_node = NULL)
    
    # Set up reactive table
    edges <- shiny::reactive({
      if (is.data.frame(category_networks[[input$category]])) {
        if (nrow(category_networks[[input$category]]) != 0) {
          edges <- category_networks[[input$category]]
          edges <- edges[edges[, sig_var] < input$slider, ]
          edges$id <- rownames(edges)
          
          if (deggs_object[["padj_method"]] != "none") {
            edges$`p adj` <- formatC(edges$p.adj, format = "e", digits = 3)
            edges <- edges[order(edges$p.adj), ]
            edges <- edges[, which(colnames(edges) != "p.value")]
          } else {
            edges$`p value` <- formatC(edges$p.value, format = "e", digits = 3)
            edges <- edges[order(edges$p.value), ]
          }
          
          if (length(input$current_edges_selection) == 0) (
            DT::datatable(edges,
                          options = list(
                            lengthChange = FALSE, scrollX = TRUE,
                            columnDefs = list(list(
                              visible = FALSE,
                              targets = c(which(names(edges) == "id") - 1,
                                          which(names(edges) == sig_var) - 1)
                            ))
                          ),
                          rownames = FALSE
            )
          ) else (
            DT::datatable(
              edges[edges$id %in% input$current_edges_selection, ],
              options = list(
                lengthChange = FALSE, scrollX = TRUE,
                columnDefs = list(list(
                  visible = FALSE,
                  targets = c(which(names(edges) == "id") - 1,
                              which(names(edges) == sig_var) - 1)
                ))
              ),
              rownames = FALSE
            )
          )
        }
      }
    })
    
    # Network
    output$network <- visNetwork::renderVisNetwork({
      if (is.data.frame(category_networks[[input$category]])) {
        if (nrow(category_networks[[input$category]]) != 0) {
          edges <- category_networks[[input$category]]
          edges$id <- rownames(edges)

          # Set up tooltip
          prefix <- ifelse(deggs_object[["padj_method"]] == "none",
                           "P=", "Padj=")
          edges$title <- paste0(prefix, formatC(edges[, sig_var], format = "e",
                                                digits = 2))

          # Set up edges width
          # normalise p value between 0 and 1
          edges$width <- (edges[, sig_var] - min(edges[, sig_var])) /
            (max(edges[, sig_var]) - min(edges[, sig_var]))
          # invert values (and multiply by 4 to increase width)
          edges$width <- (1 - edges$width) * 4

          # Set up edges color
          if (multiOmic) {
            col <- my_palette(n = nlevels(edges$layer))
            edges$color <- col[edges$layer]  # edges$layer is factor
            legend <- levels(edges$layer)
          } else {
            col <- c("#00008B", "#9A9A9A")
            edges$color <- ifelse(edges[, sig_var] < 0.05, "#00008B", "#9A9A9A")
            legend <- c("significant", " not significant")
          }

          # Slider
          edges <- edges[edges[, sig_var] < input$slider, ]
          if (nrow(edges) == 0) {
            nodes <- data.frame()
            edges <- data.frame()
            network.title <- paste0("No differential interaction with ",
                                    sig_var, "<", input$slider,
                                    ".<br> Try to increase the ",
                                    sig_var, " threshold.")
            visNetwork::visNetwork(nodes, edges, main = network.title)
          } else {
            nodes <- data.frame(
              "id" = unique(c(edges$from, edges$to)),
              "label" = unique(c(edges$from, edges$to)),
              "title" = unique(c(edges$from, edges$to))
            )

            visNetwork::visNetwork(nodes, edges) %>%
              visNetwork::visIgraphLayout(physics = TRUE,
                                          smooth = TRUE,
                                          type = "full") %>%
              visNetwork::visNodes(color = list("border" = 'white'),
                                   font = list("size" = 16)) %>%
              visNetwork::visEdges(arrows = "to",
                                   color = list("inherit" = FALSE)) %>%
              visNetwork::visLayout(randomSeed = 12) %>%
              visNetwork::visLegend(
                useGroups = FALSE,
                position = 'right',
                addEdges = data.frame(
                  color = col,
                  label = legend,
                  font.align = "top",
                  font.size = 12
                ),
                stepY = stepY_legend,
                width = legend.arrow.width,
                zoom = TRUE
              ) %>%
              visNetwork::visOptions(highlightNearest = TRUE) %>%
              visNetwork::visInteraction(hover = TRUE, tooltipDelay = 20) %>%
              visNetwork::visEvents(
                select = "function(data) {
                Shiny.onInputChange('current_nodes_selection', data.nodes);
                Shiny.onInputChange('current_edges_selection', data.edges);
                }"
              )
          }
        } else {
          network.title <- "No differential interaction active for this category"
          nodes <- data.frame()
          edges <- data.frame()
          visNetwork::visNetwork(nodes, edges, main = network.title)
        }
      } else {
        network.title <- "No differential interaction active for this category"
        nodes <- data.frame()
        edges <- data.frame()
        visNetwork::visNetwork(nodes, edges, main = network.title)
      }
    })

    # Table
    output$tbl <- DT::renderDT({
      edges()
    })
    
    # Side Plot
    output$edge_or_node_plot <- shiny::renderPlot({
      edges <- category_networks[[input$category]]
      try(
        if (is.null(input$current_nodes_selection) &
            length(input$current_edges_selection) == 1) {

          plot_regressions(
            deggs_object = deggs_object,
            gene_A = edges[input$current_edges_selection, "from"],
            gene_B = edges[input$current_edges_selection, "to"],
            assayDataName = ifelse(multiOmic, 
                   as.character(edges[input$current_edges_selection, "layer"]),
                   1)
          )
        } else {
          shiny::req(input$current_nodes_selection != "")
          node_boxplot(
            deggs_object = deggs_object,
            gene = input$current_nodes_selection,
            assayDataName = 1
          )
        }
        ,silent = TRUE)
    })
    
    # Highligh the searched node in the network
    shiny::observe({
      if (is.data.frame(category_networks[[input$category]])) {
        if (input$searchButton > 0) {
          shiny::isolate({
            edges <- category_networks[[input$category]]
            nodes <- data.frame(
              "id" = unique(c(edges$from, edges$to)),
              "label" = unique(c(edges$from, edges$to)),
              "title" = unique(c(edges$from, edges$to))
            )
            
            nodes_selection$current_node <- nodes[grep(input$searchText, 
                                                       nodes$label,
                                                       ignore.case = TRUE),
                                                  "id"]
            visNetwork::visNetworkProxy("network") %>%
              visNetwork::visSelectNodes(id = nodes_selection$current_node)
          })
        }
      }
    })
    
    shiny::observe({
      shiny::updateSliderInput(session,
                               inputId = "slider",
                               max = max(round(outVar()[, sig_var]+ 0.001,
                                               digits = 3))
      )
    })
  }
  
  ################# UI ########################
  ui <- shiny::fluidPage(
    shiny::titlePanel("Differential Networks"),
    shiny::sidebarLayout(
      shiny::sidebarPanel(
        width = 4, # this will leave more space for the network
        
        # dropdown menu for category
        shiny::selectInput(
          inputId = "category", label = "Category",
          choices = levels(deggs_object[["metadata"]]),
          selected = levels(deggs_object[["metadata"]])[1]
        ),
        
        # Slider
        shiny::sliderInput("slider",
                           label = ifelse(
                             deggs_object[["padj_method"]] == "none",
                             "P values",
                             "Adjusted P values"),
                           min = 0.01,
                           max = 10, # fake, it will be updated by updateSliderInput
                           value = 0.05, step = 0.01
        ),
        
        # Table
        shiny::tags$div(DT::DTOutput('tbl'), style = "font-size: 75%"),
        
        # Searchbox
        shinydashboard::sidebarSearchForm(
          textId = "searchText",
          buttonId = "searchButton",
          label = "Search node..."
        ),
        
        # Plots
        shiny::tags$div(shiny::plotOutput('edge_or_node_plot'))
      ),
      shiny::mainPanel(
        visNetwork::visNetworkOutput("network", height = "700px")
      )
    ),
    shiny::tags$script(shiny::HTML("$(function() {
              $('#searchText').keypress(function(e) {
              if (e.which == 13) {
                $('#searchButton').click();
              }
              });
    });"))
  )
  shiny::shinyApp(ui = ui, server = server)
}

test_that("test get_multiOmics_diffNetworks multi_omics", {
  # let's prepare a single omic example
  data("synthetic_metadata")
  data("synthetic_rnaseqData")
  data("synthetic_OlinkData")
  
  assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                         "Olink" = synthetic_OlinkData)
  
  deggs_object <- get_diffNetworks(assayData = assayData_list,
                                   metadata = synthetic_metadata,
                                   category_variable = "response",
                                   regression_method = "rlm",
                                   padj_method = "BH",
                                   verbose = FALSE,
                                   show_progressBar = FALSE,
                                   cores = 1)

  expect_silent(plot_regressions(deggs_object,
                   assayDataName = "RNAseq",
                   gene_A = "MTOR", 
                   gene_B = "AKT2",
                   legend_position = "bottomright"))
  
  expect_type(plot_regressions(deggs_object,
                                 assayDataName = "RNAseq",
                                 gene_A = "MTOR", 
                                 gene_B = "AKT2",
                                 legend_position = "bottomright"), "list")
  
  expect_no_warning(plot_regressions(deggs_object,
                                 assayDataName = "RNAseq",
                                 gene_A = "MTOR", 
                                 gene_B = "AKT2",
                                 legend_position = "bottomright"))
})
  
test_that("plot on graphical device", {
  skip_on_cran()  
  skip_on_ci()   
  skip_if_not(interactive())  
  
  expect_true({
    data("synthetic_metadata")
    data("synthetic_rnaseqData")
    data("synthetic_OlinkData")
    
    assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                           "Olink" = synthetic_OlinkData)
    
    deggs_object <- get_diffNetworks(assayData = assayData_list,
                                     metadata = synthetic_metadata,
                                     category_variable = "response",
                                     regression_method = "rlm",
                                     padj_method = "BH",
                                     verbose = FALSE,
                                     show_progressBar = FALSE,
                                     cores = 1)
    plot_regressions(deggs_object,
                     assayDataName = "RNAseq",
                     gene_A = "MTOR", 
                     gene_B = "AKT2",
                     legend_position = "bottomright")
    length(dev.list()) >= 1
  })
})

test_that("test View_diffNetworks multi_omics", {
  # let's prepare a multi-omic example
  data("synthetic_metadata")
  data("synthetic_rnaseqData")
  data("synthetic_OlinkData")
  
  assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                         "Olink" = synthetic_OlinkData)
  
  deggs_object <- get_diffNetworks(assayData = assayData_list,
                                   metadata = synthetic_metadata,
                                   category_variable = "response",
                                   regression_method = "rlm",
                                   padj_method = "BH",
                                   verbose = FALSE,
                                   show_progressBar = FALSE,
                                   cores = 1)
  
  res <- View_diffNetworks(deggs_object)
  
  expect_s3_class(res, "shiny.appobj")
  expect_type(res, "list")
  expect_true(all(c("httpHandler", "serverFuncSource", "onStart", "options") %in% 
                names(res)))
  
})

#' multiDEGGs_filter
#' 
#' Function to be passed to the `modifyX` parameter of 
#' \code{nestcv.train()} or \code{nestcv.glmnet()} (package \pkg{nestedcv}) to allow 
#' nested feature selection and augmentation via differential network analysis 
#' with multiDEGGs.
#' 
#' @param y Numeric vector or factor. Response variable (outcome), i.e. 
#'  the 'metadata' named vector, as passed by \code{nestcv.train()} or 
#'  \code{nestcv.glmnet()} (package \pkg{nestedcv}).
#' @param x Predictor variables, i.e. the assayData matrix with genes in columns
#'  and IDs in rows, as passed by \code{nestcv.train()} or
#'   \code{nestcv.glmnet()} (package \pkg{nestedcv}).
#' @param keep_single_genes Logical, default FALSE. If TRUE, the function will 
#' return unique individual genes along with significant pairs. 
#' @param nfilter Integer. Maximum total number of predictors to return. 
#' When `keep_single_genes = TRUE`, this parameter limits the combined count of 
#' unique and paired predictors
#' (i.e., length(keep_DEGGs) + nrow(pairs) <= nfilter). 
#' Predictors are included from most to least significant: for each pair, both 
#' the pair itself and the new unique variables are included until the `nfilter`
#' threshold is reached. 
#' When `keep_single_genes = FALSE`, `nfilter` only limits the number of pairs 
#' returned. 
#' If `NULL`, no filtering is applied and the total number of predictors 
#' will depend on how many significantly different interactions are detected by
#' multiDEGGs in that fold.
#'  
#' @returns a list containing two types of predictors:
#'  - single predictors (stored in the 'keep_DEGGs' vector) 
#'  - paired predictors (stored in the 'pairs' dataframe)
#' Note that `nfilter` limits the maximum number of engineered features returned,
#' however this number might be lower and will depend on how many significantly 
#' different interactions will be found in each fold by multiDEGGs.
#' \bold{If no significantly different interactions are found
#' the function will print a '0' and switch to t-test for that fold.}
#' 
#' @examples
#' \dontrun{
#' library(nestedcv)
#' data("synthetic_metadata")
#' data("synthetic_rnaseqData")
#' 
#' # fit a regularized linear model
#' # Note that nfilter, n_outer_folds, n_inner_folds are set low to keep the
#' # example lightweight. Adjust these values as needed for your use case.
#' fit.glmnet <- nestcv.glmnet(
#'   y = as.numeric(synthetic_metadata$response),
#'   x =  t(synthetic_rnaseqData),
#'   modifyX = "multiDEGGs_filter",
#'   modifyX_options = list(keep_single_genes = FALSE,
#'                          nfilter = 5),
#'   modifyX_useY = TRUE,
#'   n_outer_folds = 4,
#'   n_inner_folds = 4)
#' 
#' summary(fit.glmnet)
#' 
#' # fit a random forest model:
#' # note that nfilter, n_outer_folds, n_inner_folds are set low to keep the
#' # example lightweight. Adjust these values as needed for your use case.
#' fit.rf <- nestcv.train(
#'   y = synthetic_metadata$response,
#'   x = t(synthetic_rnaseqData),
#'   method = "rf",
#'   modifyX = "multiDEGGs_filter",
#'   modifyX_options = list(keep_single_genes = FALSE,
#'                          nfilter = 5),
#'   modifyX_useY = TRUE,
#'   n_outer_folds = 2,
#'   n_inner_folds = 2
#' )
#' 
#' fit.rf$summary
#' }
#' @export
multiDEGGs_filter <- function(y,
                              x, 
                              keep_single_genes = FALSE,
                              nfilter = NULL) {
  counts <- t(x)
  names(y) <- colnames(counts)
  deggs_object <- get_diffNetworks(assayData = counts,
                                   metadata = y,
                                   regression_method = "lm",
                                   percentile_vector = seq(0.25, 0.98, by = 0.05),
                                   padj_method = "q.value", 
                                   show_progressBar = FALSE,
                                   verbose = FALSE,
                                   cores = 1)
  
  pairs <- get_sig_deggs(deggs_object, 1, 0.05)
  
  if(keep_single_genes) {
    if (!is.null(nfilter)) {
      # Process pairs in order of significance 
      # and collect elements until nfilter is reached
      keep_DEGGs <- c()
      selected_pairs_idx <- c()
      total_elements <- 0
      
      for (i in 1:nrow(pairs)) {
        if (total_elements >= nfilter) break
        
        # Get genes from current pair
        current_genes <- c(pairs[i, 1], pairs[i, 2])
        new_genes <- setdiff(current_genes, keep_DEGGs)
        
        # Check if we can add this pair (1 element) plus any new genes
        elements_to_add <- 1 + length(new_genes)  # 1 pair + new genes
        
        if (total_elements + elements_to_add <= nfilter) {
          # Add the pair and new genes
          selected_pairs_idx <- c(selected_pairs_idx, i)
          keep_DEGGs <- c(keep_DEGGs, new_genes)
          total_elements <- total_elements + elements_to_add
        } else {
          # Check if we can add only the pair (if both genes already exist)
          if (length(new_genes) == 0 && total_elements + 1 <= nfilter) {
            selected_pairs_idx <- c(selected_pairs_idx, i)
            total_elements <- total_elements + 1
          } else {
            # Check if we can add only some new genes (without the pair)
            remaining_slots <- nfilter - total_elements
            if (remaining_slots > 0 && length(new_genes) > 0) {
              genes_to_add <- new_genes[1:min(remaining_slots,
                                              length(new_genes))]
              keep_DEGGs <- c(keep_DEGGs, genes_to_add)
              total_elements <- total_elements + length(genes_to_add)
            }
            break
          }
        }
      }
      
      # Filter pairs to only selected indices
      pairs <- pairs[selected_pairs_idx, , drop = FALSE]
    } else {
      # No nfilter applied, extract all unique genes from pairs
      keep_DEGGs <- unique(unlist(lapply(1:nrow(pairs), function(i)(
        row_n = c(pairs[i,1], pairs[i,2])
      ))))
    }
  } else {
    keep_DEGGs <- c()
    if (!is.null(nfilter)) {
      # When keep_single_genes is FALSE, nfilter only limits pairs
      if(nrow(pairs) > nfilter){
        pairs <- pairs[1:nfilter, ]
      }
    }
  }

  selected_features <- list(keep = keep_DEGGs, pairs = pairs)

  # If no differential interaction is found, default to t-test filter
  if(nrow(selected_features$pairs) <= 1){
    cat_parallel("0")
    keep_ttest <- nestedcv::ttest_filter(y, x, nfilter = nfilter, 
                                         type = "names")
    selected_features <- list(keep = keep_ttest, pairs = data.frame())
  }
  
  class(selected_features) <- "multiDEGGs_filter"
  return(selected_features)
}


##' Combined multiDEGGs filter
#'
#' This function can be passed to the `modifyX` parameter of 
#' \code{nestcv.train()} or \code{nestcv.glmnet()} (package \pkg{nestedcv})
#' to use one of the available statistical filters (t-test, wilcoxon, etc.) in 
#' combination with multiDEGGs. 
#' Single predictors will be selected by the selected statistical filter an 
#' paired predictors will be added by multiDEGGs. 
#' 
#' @param y Numeric vector or factor. Response variable (outcome), i.e. 
#'  the 'metadata' named vector, as passed by \code{nestcv.train()} or 
#'  \code{nestcv.glmnet()} (package \pkg{nestedcv}).
#' @param x Predictor variables, i.e. the assayData matrix with genes in columns
#'  and IDs in rows, as passed by \code{nestcv.train()} or
#'   \code{nestcv.glmnet()} (package \pkg{nestedcv}).
#' @param filter_method Character string. Statistical filtering method to be
#' used in combination with multiDEGGs for sigle feature selection. 
#' Options are: "ttest", "wilcoxon", "ranger", "glmnet", "pls".
#' @param nfilter Integer. Maximum number of features to select. 
#' @param dynamic_nfilter Logical. If `TRUE` `nfilter` will limit the number of 
#' features selected by the statistical filter and the feature space will be
#' augmented by adding ALL the paired predictors found by multiDEGGs.
#' If `FALSE` `nfilter` will limit the total number of predictors, with 
#' approximately half allocated to pairs and half to single genes.
#' @param keep_single_genes Logical. When `dynamic_nfilter = TRUE`, determines
#'   whether to include single genes selected by multiDEGGs (i.e. the single 
#'   variables included in the differential pairs) in addition to those from
#'   the statistical filter. Default is FALSE.
#' @param ... Additional arguments passed to the filtering functions.
#'
#' @details
#' The function operates in two modes:
#'
#' \strong{Dynamic Filtering (dynamic_nfilter = TRUE):}
#' - Selects \code{nfilter} single genes using the specified statistical method
#' - Finds all significant gene pairs using multiDEGGs
#' - Total predictors = nfilter single genes + number of significant pairs
#'    - If \code{keep_single_genes = TRUE}, also includes single genes obtained 
#'      from pairs found by multiDEGGs
#'
#' \strong{Balanced Selection (dynamic_nfilter = FALSE):}
#' - Allocates approximately half of \code{nfilter} to gene pairs
#' - Remaining slots filled with single genes from the statistical filter
#' - If fewer pairs are found than allocated, compensates by selecting more 
#'   single genes
#' - Ensures consistent total number of predictors across outer folds
#'
#' The statistical filtering methods include:
#' - \code{"ttest"}: Two-sample t-test for differential expression
#' - \code{"wilcoxon"}: Wilcoxon rank-sum test
#' - \code{"ranger"}: Random Forest variable importance
#' - \code{"glmnet"}: Elastic net regularization
#' - \code{"pls"}: Partial Least Squares variable importance
#'
#' @return An object of class "multiDEGGs_filter" containing:
#' \item{keep}{Character vector of selected single gene names}
#' \item{pairs}{Data frame of selected gene pairs with interaction information}
#' @examples
#' \dontrun{
#' library(nestedcv)
#' data("synthetic_metadata")
#' data("synthetic_rnaseqData")
#' 
#' # fit a regularized linear model
#' # note that nfilter, n_outer_folds, n_inner_folds are set low to keep the
#' # example lightweight. Adjust these values as needed for your use case.
#' fit.glmnet <- nestedcv::nestcv.glmnet(
#' y = as.numeric(synthetic_metadata$response),
#' x =  t(synthetic_rnaseqData),
#' modifyX = "multiDEGGs_combined_filter",
#' modifyX_options = list(filter_method = "ttest", 
#'                        nfilter = 5,
#'                        dynamic_nfilter = TRUE, 
#'                        keep_single_genes = FALSE),
#' modifyX_useY = TRUE,
#' n_outer_folds = 4,
#' n_inner_folds = 4)
#' 
#' summary(fit.glmnet)
#' 
#' # fit a random forest model
#' # NOTE: nfilter, n_outer_folds, n_inner_folds are set low to keep the
#' # example lightweight. Adjust these values as needed for your use case.
#' fit.rf <- nestedcv::nestcv.train(
#'   y = synthetic_metadata$response,
#'   x = t(synthetic_rnaseqData),
#'   method = "rf",
#'   modifyX = "multiDEGGs_combined_filter",
#'   modifyX_options = list(filter_method = "ttest", 
#'                          nfilter = 5,
#'                          dynamic_nfilter = TRUE, 
#'                          keep_single_genes = FALSE),
#'   modifyX_useY = TRUE,
#'   n_outer_folds = 2,
#'   n_inner_folds = 2
#' )
#' 
#' fit.rf$summary
#' }
#' @export
multiDEGGs_combined_filter <- function(y,
                                       x,
                                       filter_method = "ttest", 
                                       nfilter,
                                       dynamic_nfilter = TRUE, 
                                       keep_single_genes = FALSE, 
                                       ...) {
  if (dynamic_nfilter) {
    # The total number of predictors will depend on how many sig interactions 
    # are found by multiDEGGs, so the tot number of predictors will be 
    # nfilter + pairs in each outer fold
    keep_filtered <- switch(filter_method,
                            "ttest" = nestedcv::ttest_filter(y, x, nfilter = nfilter,
                                                   type = "names"),
                            "wilcoxon" = nestedcv::wilcoxon_filter(y, x, nfilter = nfilter,
                                                         type = "names"),
                            "ranger" = nestedcv::ranger_filter(y, x, nfilter = nfilter, 
                                                     type = "names"),
                            "glmnet" = nestedcv::glmnet_filter(y, x, nfilter = nfilter, 
                                                     type = "names"),
                            "pls" = nestedcv::pls_filter(y, x, nfilter = nfilter, 
                                               type = "names"),
                            stop("Invalid filter_method. 
                                 Choose from: ttest, wilcoxon, ranger, 
                                              glmnet, pls"))
    
    multiDEGGs_res <- multiDEGGs_filter(y, x, keep_single_genes = TRUE)

    if (keep_single_genes) {
      # select single genes from both multiDEGGs and chosen filter
      selected_features <- list(keep = c(keep_filtered, 
                                         multiDEGGs_res$keep_DEGGs), 
                                pairs = multiDEGGs_res$pairs)
    } else {
      # single predictors from filter_method, pairs from multiDEGGs
      selected_features <- list(keep = keep_filtered,
                                pairs = multiDEGGs_res$pairs)
    }
  } else {
    # balanced selection, no single genes selected by multiDEGGs (only pairs)
    n_pairs <- floor(nfilter / 2)   # max half for interactions
    n_single <- nfilter - n_pairs   # remaining for single genes
    
    # Get significant pairs from multiDEGGs first (no filter on number)
    multiDEGGs_res <- multiDEGGs_filter(y, x, nfilter = NULL,
                                        keep_single_genes = FALSE)
    
    # Select available pairs (up to n_pairs)
    if (nrow(multiDEGGs_res$pairs) > 0) {
      selected_pairs <- multiDEGGs_res$pairs[1:min(n_pairs, 
                                                   nrow(multiDEGGs_res$pairs)),,
                                             drop = FALSE]
      actual_pairs <- nrow(selected_pairs)
    } else {
      selected_pairs <- data.frame()
      actual_pairs <- 0
    }
    
    # Calculate how many single genes we need from selected filter 
    # (compensate for missing pairs to have an even number of predictors in each
    # outer fold)
    singles_needed <- n_single + (n_pairs - actual_pairs)
    
    # Get single genes from selected filter
    keep_filtered <- switch(filter_method,
                            "ttest" = nestedcv::ttest_filter(y, x,
                                                   nfilter = singles_needed,
                                                   type = "names"),
                            "wilcoxon" = nestedcv::wilcoxon_filter(y, x, 
                                                    nfilter = singles_needed,
                                                    type = "names"),
                            "ranger" = nestedcv::ranger_filter(y, x, 
                                                     nfilter = singles_needed, 
                                                     type = "names"),
                            "glmnet" = nestedcv::glmnet_filter(y, x, 
                                                     nfilter = singles_needed,
                                                     type = "names"),
                            "pls" = nestedcv::pls_filter(y, x, 
                                               nfilter = singles_needed, 
                                               type = "names"),
                            stop("Invalid filter_method. 
                                 Choose from: ttest, wilcoxon, 
                                 ranger, glmnet, pls")
    )
    selected_features <- list(keep = keep_filtered, pairs = selected_pairs)
  }
  class(selected_features) <- "multiDEGGs_filter"
  return(selected_features)
}


#' Predict method for multiDEGGs_filter objects
#'
#' This function generates predictions by creating a dataset with single 
#' and combined predictors based on the filtering results of a
#' multiDEGGs_filter model.
#'
#' @param object A fitted object of class `multiDEGGs_filter` containing
#'  filtering results with:
#'   \describe{
#'     \item{keep}{Character vector of variable names to keep as single 
#'     predictors}
#'     \item{pairs}{Data frame or matrix with two columns specifying pairs 
#'     of variables to combine}
#'   }
#' @param newdata A data frame containing the new data for prediction. 
#' Must contain all variables specified in \code{object$keep} and
#' \code{object$pairs}.
#' @param interaction.type Character string specifying how to combine the 
#' paired predictors.
#'   Options are:
#'   \describe{
#'     \item{"ratio"}{Combine paired predictors by dividing the first variable
#'      by the second (a/b)}
#'     \item{other}{Combine paired predictors by multiplying the variables (a*b)
#'     }
#'   }
#'   Default is "ratio".
#' @param sep Character string used as separator when creating column names for
#'   combined predictors. Default is ":".
#' @param ... Additional arguments passed to the generic function.
#'
#' @return A data frame containing:
#'   \itemize{
#'     \item Single predictors (if any are specified in \code{object$keep})
#'     \item Combined predictors based on variable pairs and interaction type
#'   }
#'
#' @details
#' The function processes the filtering results in two steps:
#' \enumerate{
#'   \item Selects single predictors from \code{newdata} based on variables 
#'   listed in \code{object$keep}
#'   \item Adds combined predictors from paired variables in
#'    \code{object$pairs}
#' }
#' @export
.predict_multiDEGGs <- function(object,
                                newdata,
                                interaction.type = "ratio",
                                sep = ":",
                                ...) {
  result_data <- NULL
  keep <- object$keep
  
  # Add single predictors if present
  if(length(keep) > 0){
    single_preds <- newdata[, keep, drop = FALSE]
    result_data <- single_preds
  }
  
  # Add combined predictors if present
  if (nrow(object$pairs) > 0) {
    pairs <- object$pairs
    a <- newdata[, pairs[, 1], drop = FALSE]
    b <- newdata[, pairs[, 2], drop = FALSE]
    
    if (interaction.type == "ratio") {
      combined_preds <- a/b
    } else {
      combined_preds <- a*b
    }
    combined_preds[!is.finite(combined_preds)] <- 0
    colnames(combined_preds) <- paste(colnames(a), colnames(b), sep = sep)
    
    if (is.null(result_data)) {
      result_data <- combined_preds
    } else {
      result_data <- cbind(result_data, combined_preds)
    }
  }

  if (is.null(result_data)) {
    stop("The custom filter is empty")
  }
  return(result_data)
}


#' Wrapper of .predict_multiDEGGs for multiDEGGs_filter()
#'
#' This function generates predictions by creating a dataset with single 
#' and combined predictors based on the filtering results of a
#' multiDEGGs_filter model.
#'
#' @inherit .predict_multiDEGGs
#' @export
predict.multiDEGGs_filter <- function(object,
                                      newdata,
                                      interaction.type = "ratio",
                                      sep = ":", 
                                      ...) {
  .predict_multiDEGGs(object, newdata, interaction.type = "ratio",
                      sep = ":", ...)
}


#' Wrapper of .predict_multiDEGGs for multiDEGGs_filter_combined()
#'
#' This function generates predictions by creating a dataset with single 
#' and combined predictors based on the filtering results of a
#' multiDEGGs_filter model.
#'
#' @inherit .predict_multiDEGGs
#' @export
predict.multiDEGGs_filter_combined <- function(object,
                                               newdata,
                                               interaction.type = "ratio",
                                               sep = ":", 
                                               ...) {
  .predict_multiDEGGs(object, newdata, interaction.type = "ratio",
                      sep = ":", ...)
}


#' cat_parallel (from nestedcv)
#' 
#' Prints using shell echo from inside mclapply when run in Rstudio
#' @param \dots to be passed to system()
cat_parallel <- function(...) {
  if (Sys.getenv("RSTUDIO") != "1") return()
  system(sprintf('echo "%s', paste0(..., '\\c"', collapse = "")))
}

# multiDEGGs_filter - test -------------------------------------------------
test_that("test multiDEGGs_filter", {
  skip_if_not_installed("nestedcv")
  set.seed(123)
  data("synthetic_metadata")
  data("synthetic_rnaseqData")

  res <- multiDEGGs_filter(y = as.numeric(synthetic_metadata$response),
                    x =  t(synthetic_rnaseqData))
  
  expect_s3_class(res, "multiDEGGs_filter")
  expect_equal(ncol(res$pairs), 4)
  expect_equal(nrow(res$pairs), 7)
  expect_true("IL1B" %in% res$pairs$from)
  expect_true("IL1R2" %in% res$pairs$to)
  expect_true("FASLG" %in% res$pairs$from)
  expect_true("FAS" %in% res$pairs$to)
  
  fit.glmnet <- nestedcv::nestcv.glmnet(
    y = as.numeric(synthetic_metadata$response),
    x =  t(synthetic_rnaseqData),
    modifyX = "multiDEGGs_filter",
    modifyX_options = list(keep_single_genes = FALSE,
                           nfilter = 20),
    modifyX_useY = TRUE,
    n_outer_folds = 5,
    n_inner_folds = 6)
  
  fit.glmnet.summary <- summary(fit.glmnet)
  
  expect_true("TNF:TNFRSF1A" %in% names(fit.glmnet.summary[["coef"]]))
  
  fit.svm <- nestedcv::nestcv.train(
    y = synthetic_metadata$response,
    x = t(synthetic_rnaseqData),
    method = "svmRadial",
    modifyX = "multiDEGGs_filter",
    modifyX_options = list(keep_single_genes = TRUE,
                           nfilter = 6),
    modifyX_useY = TRUE,
    n_outer_folds = 5,
    n_inner_folds = 6,
    verbose = FALSE
  )
  
  expect_s3_class(fit.svm, "nestcv.train")
  expect_equal(length(fit.svm[["final_vars"]]), 6)
})

# multiDEGGs_combined_filter - test -------------------------------------------------
test_that("test multiDEGGs_combined_filter", {
  skip_if_not_installed("nestedcv")
  set.seed(123)
  data("synthetic_metadata")
  data("synthetic_rnaseqData")
  
  res <- multiDEGGs_combined_filter(y = as.numeric(synthetic_metadata$response),
                           x =  t(synthetic_rnaseqData), 
                           filter_method = "ttest",
                           nfilter = 10,
                           dynamic_nfilter = FALSE)
  
  expect_s3_class(res, "multiDEGGs_filter")
  expect_true("MAPK3" %in% res[["keep"]])
  expect_true("TGFB3" %in% res[["keep"]])
  expect_equal(ncol(res$pairs), 4)
  expect_equal(nrow(res$pairs), 5)
  expect_true("FANCD2" %in% res$pairs$from)
  expect_true("FAN1" %in% res$pairs$to)
  
  res2 <- multiDEGGs_combined_filter(y = as.numeric(synthetic_metadata$response),
                                    x =  t(synthetic_rnaseqData), 
                                    filter_method = "wilcoxon",
                                    nfilter = 10,
                                    dynamic_nfilter = TRUE)
  
  
  expect_s3_class(res2, "multiDEGGs_filter")
  expect_equal(length(res2[["keep"]]), 10)
  expect_true(all(c("TNF", "MAP2K2", "MAPK3", "TNFRSF1A", "MTOR", "FAN1",
                    "IL1B") %in% res2[["keep"]]))
  expect_equal(ncol(res2$pairs), 4)
  expect_equal(nrow(res2$pairs), 7)
  expect_true("IL1B" %in% res2$pairs$from)
  expect_true("IL1R2" %in% res2$pairs$to)
  
  res3 <- multiDEGGs_combined_filter(y = as.numeric(synthetic_metadata$response),
                                     x =  t(synthetic_rnaseqData), 
                                     filter_method = "pls",
                                     nfilter = 10,
                                     dynamic_nfilter = TRUE,
                                     keep_single_genes = TRUE)
  
  expect_s3_class(res3, "multiDEGGs_filter")
  expect_equal(length(res3[["keep"]]), 10)
  expect_true(all(c("MAP2K2", "TNFRSF1A", "MTOR", "FANCD2", "AKT2", 
                    "FAN1" ) %in% res3[["keep"]]))
  expect_equal(ncol(res3$pairs), 4)
  expect_equal(nrow(res3$pairs), 7)
  expect_true("IL1B" %in% res3$pairs$from)
  expect_true("TNFRSF1A" %in% res3$pairs$to)
  
  fit.glmnet <- nestedcv::nestcv.glmnet(
    y = as.numeric(synthetic_metadata$response),
    x =  t(synthetic_rnaseqData),
    modifyX = "multiDEGGs_combined_filter",
    modifyX_options = list(filter_method = "ttest", 
                           nfilter = 20,
                           dynamic_nfilter = TRUE, 
                           keep_single_genes = FALSE),
    modifyX_useY = TRUE,
    n_outer_folds = 5,
    n_inner_folds = 6)
  
  fit.glmnet.summary <- summary(fit.glmnet)
  
  expect_equal(length(fit.glmnet[["final_vars"]]), 21)
  expect_equal(nrow(fit.glmnet[["final_coef"]]), 15)
  
  expect_true("MAP2K2:MAPK3" %in% fit.glmnet[["final_vars"]])
  
  fit.rf <- nestedcv::nestcv.train(
    y = synthetic_metadata$response,
    x = t(synthetic_rnaseqData),
    method = "rf",
    modifyX = "multiDEGGs_combined_filter",
    modifyX_options = list(filter_method = "ttest", 
                           nfilter = 5,
                           dynamic_nfilter = TRUE, 
                           keep_single_genes = FALSE),
    modifyX_useY = TRUE,
    n_outer_folds = 5,
    n_inner_folds = 6,
    verbose = FALSE
  )
  
  expect_s3_class(fit.rf, "nestcv.train")
  expect_equal(fit.rf[["summary"]][["metrics"]][["AUC"]], 0.99, 
               tolerance = 0.01)
  expect_equal(fit.rf[["summary"]][["metrics"]][["Accuracy"]], 0.98, 
               tolerance = 0.01)
  expect_equal(length(fit.rf[["final_vars"]]), 12)
})

#' Generate multi-omic differential networks
#'
#' Generate a multi-layer differential network with interaction p values
#'
#' @param assayData a matrix or data.frame (or list of matrices or data.frames
#' for multi-omic analysis) containing normalised assay data.
#' Sample IDs must be in columns and probe IDs (genes, proteins...) in rows.
#' For multi omic analysis, it is highly recommended to use a named list of
#' data. 
#' If unnamed, sequential names (assayData1, assayData2, etc.) will be
#' assigned to identify each matrix or data.frame. 
#' @param metadata a named vector, matrix, or data.frame containing sample 
#' annotations or categories. If matrix or data.frame, each row should
#' correspond to a sample, with columns representing different sample 
#' characteristics (e.g., treatment group, condition, time point). The colname 
#' of the sample characteristic to be used for differential analysis must be 
#' specified in `category_variable`. Rownames must match the sample IDs used in
#' assayData. 
#' If named vector, each element must correspond to a sample characteristic to 
#' be used for differential analysis, and names must match sample IDs used in
#' the colnames of `assayData`.
#' Continuous variables are not allowed.  
#' @param category_variable when metadata is a matrix or data.frame this is the 
#' column name of `metadata` that contains the sample annotations to be used for
#' differential analysis
#' @param regression_method whether to use robust linear modelling to calculate
#' link p values. Options are 'lm' (default) or 'rlm'. The lm implementation 
#' is faster and lighter.  
#' @param category_subset optional character vector indicating a subset of 
#' categories from the category variable. If not specified, all categories in
#' `category_variable` will be used.
#' @param network network of biological interactions provided by the user. The
#' network must be provided in the form of a table of class data.frame with only 
#' two columns named "from" and "to".
#' If NULL (default) a network of 10,537 molecular interactions obtained from
#' KEGG, mirTARbase, miRecords and transmiR will be used.
#' This has been obtained via the `exportgraph` function of the MITHrIL tool
#' (Alaimo et al., 2016).
#' @param  percentile_vector a numeric vector specifying the percentiles to be
#' used in the percolation analysis. By default, it is defined as 
#' `seq(0.35, 0.98, by = 0.05)`, which generates a sequence of percentiles
#' starting at 0.35, meaning that targets (genes/proteins...) whose expression 
#' value is under the 35th percentile of the whole matrix will be excluded.
#' This threshold can be modified by specifying a different starting point for 
#' `seq`. For a more granular percolation analysis an higher optimisation of 
#' the algorithm, `by = 0.05` can be modified in favour of lower values, but 
#' this will increase the computational time.  
#' @param padj_method a character string indicating the p values correction 
#' method for multiple test adjustment. It can be either one of the methods 
#' provided by the `p.adjust` function from `stats` (bonferroni, BH, hochberg, 
#' etc.) or "q.value" for Storey's q values, or "none" for unadjusted p values.
#' When using "q.value" the `qvalue` package must be installed first.
#' @param show_progressBar logical. Whether to display a progress bar during 
#' execution. Default is TRUE.
#' @param verbose logical. Whether to print detailed output messages during 
#' processing. Default is TRUE
#' @param cores number of cores to use for parallelisation.
#' @return a `deggs` object containing differential networks incorporating
#' p values or adjusted p values for each link.
#' @examples
#' # Single omic analysis:
#' data("synthetic_metadata")
#' data("synthetic_rnaseqData")
#' deggs_object <- get_diffNetworks(assayData = synthetic_rnaseqData,
#'                                  metadata = synthetic_metadata,
#'                                  category_variable = "response",
#'                                  regression_method = "lm",
#'                                  padj_method = "bonferroni",
#'                                  verbose = FALSE,
#'                                  show_progressBar = FALSE,
#'                                  cores = 1)
#'  
#' # multi-omic analysis: 
#' data("synthetic_metadata")
#' data("synthetic_rnaseqData")
#' data("synthetic_proteomicData")
#' data("synthetic_OlinkData")
#' assayData_list <- list("RNAseq" = synthetic_rnaseqData,
#'                        "Proteomics" = synthetic_proteomicData,
#'                        "Olink" = synthetic_OlinkData)
#' deggs_object <- get_diffNetworks(assayData = assayData_list,
#'                                  metadata = synthetic_metadata,
#'                                  category_variable = "response",
#'                                  regression_method = "lm",
#'                                  padj_method = "bonferroni",
#'                                  verbose = FALSE,
#'                                  show_progressBar = FALSE,
#'                                  cores = 1)
#'                                  
#' # to use only certain categories for comparison: 
#' # let's randomly add another level of response to the example metadata
#' synthetic_metadata$response <- as.character(synthetic_metadata$response)
#' indices <- sample(1:nrow(synthetic_metadata), 20, replace = FALSE) 
#' synthetic_metadata$response[indices] <- "Moderate response"
#' deggs_object <- get_diffNetworks(assayData = assayData_list,
#'                                  metadata = synthetic_metadata,
#'                                  category_variable = "response",
#'                                  category_subset = c("Responder", 
#'                                                      "Non_responder"),
#'                                  regression_method = "lm",
#'                                  verbose = FALSE,
#'                                  show_progressBar = FALSE,
#'                                  cores = 1)
#'                                  
#' # to be more generous on the targets to be excluded, and lower the expression 
#' # level threshold to the 25th percentile (or lower): 
#' deggs_object <- get_diffNetworks(assayData = assayData_list,
#'                                  metadata = synthetic_metadata,
#'                                  category_variable = "response",
#'                                  category_subset = c("Responder", 
#'                                                      "Non_responder"),
#'                                  regression_method = "lm",
#'                                  percentile_vector = seq(0.25, 0.98, by = 0.05),
#'                                  verbose = FALSE,
#'                                  show_progressBar = FALSE,
#'                                  cores = 1)                                  
#' @export
get_diffNetworks <- function(assayData,
                             metadata,
                             category_variable = NULL,
                             regression_method = 'lm',
                             category_subset = NULL,
                             network = NULL,
                             percentile_vector = seq(0.35, 0.98, by = 0.05),
                             padj_method = "bonferroni",
                             show_progressBar = TRUE,
                             verbose = TRUE, 
                             cores = parallel::detectCores() / 2) {
  
  if (!(is.list(assayData) || 
        is.matrix(assayData) || is.data.frame(assayData))) (
          stop("assayDataList is not a list or a matrix or data.frame")
        )
  
  if (is.matrix(assayData) || is.data.frame(assayData)) (
    assayDataList <- list(assayData)
  ) else (
    assayDataList <- assayData
  )
  
  if (!is.null(category_variable)) {
    if ((is.matrix(metadata) || is.data.frame(metadata)) && 
        !category_variable %in% colnames(metadata)) {
      stop("category_variable must be %in% colnames(metadata). If metadata is a 
       named vector category_variable must be NULL.")
    }
  } else if (is.matrix(metadata) || is.data.frame(metadata)) {
    stop("metadata is a matrix or dataframe but category_variable is NULL.
     Please specify category_variable to select the correct column 
     from metadata.")
  }
  
  if (is.null(names(assayDataList))) (
    names(assayDataList) <- paste0("assayData", seq_along(assayDataList))
  )
  
  metadata <- tidy_metadata(
    category_subset = category_subset, metadata = metadata,
    category_variable = category_variable, verbose = verbose
  )  # metadata will be a named vector from now on
  
  diffNetworks_list <- lapply(seq_along(assayDataList), function(i){
    assayDataName <- names(assayDataList)[i]
    assayData <- assayDataList[[assayDataName]]
    
    diffNetworks <- 
      tryCatch({
        get_diffNetworks_singleOmic(assayData,
                                    assayDataName, 
                                    metadata,
                                    regression_method,
                                    network,
                                    percentile_vector,
                                    padj_method,
                                    show_progressBar,
                                    verbose, 
                                    cores)
      }, error = function(e) { message(conditionMessage(e)) })
  })
  names(diffNetworks_list) <- names(assayDataList)
  
  degg <- list(
    diffNetworks = diffNetworks_list,
    assayData = assayDataList,
    metadata = metadata,
    regression_method = regression_method,
    category_subset = category_subset,
    padj_method = padj_method
  )
  class(degg) <- "deggs"
  return(degg)
}

#' Generate differential networks for single omic analysis 
#' 
#' @inheritParams get_diffNetworks
#' @param assayDataName name of the assayData, to identify which omic is. 
#' @return a list of differential networks, one per category
get_diffNetworks_singleOmic <- function(assayData,
                                        assayDataName,
                                        metadata,
                                        regression_method,
                                        network,
                                        percentile_vector, 
                                        padj_method,
                                        show_progressBar,
                                        verbose, 
                                        cores) {
  
  if (verbose) (
    message("Generating ", assayDataName, " network layer...")
  )
  
  if (!(is.matrix(assayData) || is.data.frame(assayData))) (
    stop(assayDataName, " is not a matrix or data.frame")
  )
  
  if(!any(colnames(assayData) %in% names(metadata))) (
    stop("Sample IDs in ", assayDataName, " don't match the IDs in metadata 
          (the metadata names/rownames must match the sample IDs used as column 
          names of ", assayDataName, ".")
  )
  
  # check match between metadata and assayData
  if (length(intersect(colnames(assayData), names(metadata))) == 0) {
    stop("Sample IDs in metadata and ", assayDataName, " don't match.")
  }
  
  # remove assayData samples that don't exist in metadata (because they
  # were missing or because they were filtered out due to the category_subset)
  assayData <- assayData[, which(colnames(assayData) %in% names(metadata)),
                         drop = FALSE]
  # remove cols with 0 or 1 non-NA values (we need at least two points)
  assayData <- assayData[, colSums(is.na(assayData)) < (nrow(assayData) - 1)]
  
  # message on metadata sample IDs that don't exist in assayData
  missing_metadataSamples <- names(metadata)[which(!(names(metadata) %in% 
                                                       colnames(assayData)))]
  
  if (length(missing_metadataSamples) != 0) {
    missing_samples_formatted <- paste(missing_metadataSamples, collapse = ", ")
    message("The following samples IDs are missing in ", assayDataName, 
            ":\n", missing_samples_formatted)
  }
  
  # align 
  metadata <- metadata[colnames(assayData)]
  if(length(unique(metadata)) == 1) (
    stop("All sample IDs in ", assayDataName, " belong to one category. 
           No differential analysis is possible.")
  )
  
  # network
  if (!is.null(network)) (
    # user provided network
    network_to_use <- network
  ) else (
    network_to_use <- omic_network
  )
  
  edges <- network_to_use[network_to_use$from %in% rownames(assayData) & 
                            network_to_use$to %in% rownames(assayData), ] 
  
  if (nrow(edges) == 0) (
    stop("Rownames of ", assayDataName, " don't match any link of the biological
    network. Check if names need conversion to gene symbols or entrez IDs. 
    Alternatively, provide more data.")
  )
  
  # Remove duplicate edges
  edges$edge_ID <- paste(edges$from, edges$to)
  edges <- edges[!duplicated(edges$edge_ID), ]
  edges$edge_ID <- NULL
  
  nodes <- c(edges[, 1], edges[, 2]) %>%
    unique()
  
  assayData <- assayData[rownames(assayData) %in% nodes, ]
  
  categories <- levels(metadata)
  
  # pairwise contrasts are necessary for more than 2 categories  
  contrasts <- utils::combn(categories, m = 2) %>%
    as.data.frame()
  
  # create categories (duplicating assay data for each category)
  category_median_list <- lapply(categories, function(one_category) {
    # Get sample IDs belonging to the current category
    sample_ids_in_category <- names(metadata)[metadata == one_category]
    
    # Select samples from assayData using these IDs
    category_vec <- assayData[, sample_ids_in_category, drop = FALSE]
    
    # Calculate median expression across samples
    category_vec <- apply(category_vec, 1, stats::median, na.rm = TRUE)
    rownames(category_vec) <- NULL
    return(category_vec)
  })
  names(category_median_list) <- categories
  
  # Percolation analysis: 
  # calculate interaction p values per percentile and select the percentile 
  # network with the highest number of significant interactions 
  sig_edges_count <- 0
  
  if (Sys.info()["sysname"] == "Windows") {
    cl <- parallel::makeCluster(cores)
    
    parallel::clusterExport(cl, c(
      "percentile_vector", "category_median_list", "contrasts",
      "regression_method", "edges", "categories", "calc_pvalues_percentile",
      "assayData", "metadata",
      "sig_edges_count", "padj_method"
    ), envir = environment())
    
    if (show_progressBar) (
      pvalues_list <- pbapply::pblapply(
        cl = cl, percentile_vector,
        function(percentile) {
          calc_pvalues_percentile(
            assayData = assayData,
            padj_method = padj_method,
            metadata = metadata,
            percentile = percentile,
            category_median_list = category_median_list,
            contrasts = contrasts,
            regression_method = regression_method,
            edges = edges,
            categories_length = length(categories),
            sig_edges_count = sig_edges_count
          )
        })
    ) else (
      pvalues_list <- parallel::parLapply(
        cl = cl, percentile_vector,
        function(percentile) {
          calc_pvalues_percentile(
            assayData = assayData,
            padj_method = padj_method,
            metadata = metadata,
            percentile = percentile,
            category_median_list = category_median_list,
            contrasts = contrasts,
            regression_method = regression_method,
            edges = edges,
            categories_length = length(categories),
            sig_edges_count = sig_edges_count
          )
        })
    )
    parallel::stopCluster(cl)
  } else {
    # Parallelisation for mac/linux
    if (show_progressBar) (
      pvalues_list <- pbmcapply::pbmclapply(
        mc.cores = cores, percentile_vector,
        function(percentile) (
          calc_pvalues_percentile(
            assayData = assayData,
            padj_method = padj_method,
            metadata = metadata,
            percentile = percentile,
            category_median_list = category_median_list,
            contrasts = contrasts,
            regression_method = regression_method,
            edges = edges,
            categories_length = length(categories),
            sig_edges_count = sig_edges_count
          )
        )
      )
    ) else (
      pvalues_list <- parallel::mclapply(
        mc.cores = cores, percentile_vector,
        function(percentile) (
          calc_pvalues_percentile(
            assayData = assayData,
            padj_method = padj_method,
            metadata = metadata,
            percentile = percentile,
            category_median_list = category_median_list,
            contrasts = contrasts,
            regression_method = regression_method,
            edges = edges,
            categories_length = length(categories),
            sig_edges_count = sig_edges_count
          )
        )
      )
    )
    
  }
  names(pvalues_list) <- percentile_vector
  
  # Extract the network filtered at the optimal threshold percentile
  # (the one with the highest number of statistically significant interactions)
  sig_pvalues <- vapply(pvalues_list, function(networks) {
    if (!inherits(networks, "try-error") && !is.null(networks)) {
      return(networks$sig_pvalues_count)
    } else {
      return(NA_real_)
    }
  }, FUN.VALUE = numeric(1))
  
  sig_pvalues <- sig_pvalues[!is.na(sig_pvalues)]
  
  if (length(sig_pvalues) == 0) {
    stop("No significant differences detected across categories.")
  }
  
  best_percentile <- sig_pvalues[which.max(sig_pvalues)]
  if (verbose) (
    message("Percolation analysis: genes whose expression is below the ",
            as.numeric(names(best_percentile)) * 100,
            "th percentile are removed from networks.")
  )
  final_networks <- pvalues_list[[as.character(names(best_percentile))]]
  return(final_networks)
}


#' Tidying up of metadata. Samples belonging to undesidered categories
#' (if specified) will be removed as well as categories with less than five 
#' samples, and NAs.
#'
#' @param category_subset optional character vector indicating which categories
#' are used for comparison. If not specified, all categories in
#' `category_variable` will be used.
#' @param metadata a named vector, matrix, or data.frame containing sample 
#' annotations or categories. If matrix or data.frame, each row should
#' correspond to a sample, with columns representing different sample 
#' characteristics (e.g., treatment group, condition, time point). The colname 
#' of the sample characteristic to be used for differential analysis must be 
#' specified in `category_variable`. Rownames must match the sample IDs used in
#' assayData. 
#' If named vector, each element must correspond to a sample characteristic to 
#' be used for differential analysis, and names must match sample IDs used in
#' the colnames of `assayData`.
#' Continuous variables are not allowed.  
#' @param category_variable column name in `metadata` (if data.frame or matrix)
#' or NULL if `metadata` is already a named vector containing category 
#' information.
#' @param verbose Logical. Whether to print detailed output messages
#' during processing. Default is FALSE.
#' @return a tidy named factor vector of sample annotations.
tidy_metadata <- function(category_subset = NULL,
                          metadata,
                          category_variable = NULL, 
                          verbose = FALSE) {
  
  # Convert metadata to a named vector based on input type
  if (is.atomic(metadata) || is.factor(metadata)) {
    if (is.null(names(metadata))) {
      stop("Named vector required: metadata vector must have names.")
    }
    # Already a named vector
    category_vector <- metadata
  } else if (is.matrix(metadata) || is.data.frame(metadata)) {
    if (is.null(category_variable)) {
      stop("category_variable must be specified when metadata is
           a matrix or data.frame.")
    }
    if (!category_variable %in% colnames(metadata)) {
      stop("category_variable '", category_variable, "' not found in metadata.")
    }
    category_vector <- metadata[, category_variable]
    names(category_vector) <- rownames(metadata)
  } else {
    stop("metadata must be a named vector/factor, matrix, or data.frame.")
  }
  
  # Subset seleted categories (optional)
  if (!is.null(category_subset)) {
    if (!all(category_subset %in% category_vector)) (
      stop("category_subset values must be contained in metadata")
    )
    keep_samples <- names(category_vector)[category_vector %in% category_subset]
    if (length(keep_samples) == 0) {
      stop("No samples remain after filtering for specified category_subset.")
    }
    category_vector <- category_vector[keep_samples]
  }
  
  # Convert to factor
  if (!is.factor(category_vector)) {
    category_vector <- as.factor(category_vector)
    if (verbose) {
      message("category values were converted to factor")
    }
  }
  
  # Remove categories with less than five observations
  tbl <- table(category_vector)
  small_groups <- names(tbl)[tbl < 5]
  
  if (length(small_groups) > 0) {
    for (group in small_groups) {
      message("The ", group, 
              " category did not contain enough samples (less than five 
              observations). ",
              "This category will be removed.")
    }
    keep_samples <- names(category_vector)[category_vector %in%
                                             names(tbl)[tbl >= 5]]
    category_vector <- category_vector[keep_samples]
    category_vector <- droplevels(category_vector)
    
    if (nlevels(category_vector) < 2) {
      stop("At least two categories with at least 5 observations are required.")
    }
  }
  
  # Remove NAs
  NAs_number <- sum(is.na(category_vector))
  if (NAs_number > 0) {
    category_vector <- stats::na.omit(category_vector)
    if (verbose) {
      message("category values contained NAs. ",
              NAs_number, " samples were removed.")
    }
  }
  return(category_vector)
}


#' Compute interaction p values for a single percentile value
#'
#' @inheritParams get_diffNetworks
#' @param categories_length integer number indicating the number of categories
#' @param category_median_list list of category data.frames
#' @param percentile a float number indicating the percentile to use.
#' @param contrasts data.frame containing the categories contrasts in rows
#' @param edges network of biological interactions in the form of a table of
#' class data.frame with two columns: "from" and "to".
#' @param sig_edges_count number of significant edges (p < 0.05)
#' @importFrom magrittr %>%
#' @return The list of float numbers of the significant pvalues for a single
#' percentile
calc_pvalues_percentile <- function(assayData,
                                    metadata,
                                    categories_length,
                                    category_median_list,
                                    padj_method,
                                    percentile,
                                    contrasts,
                                    regression_method,
                                    edges,
                                    sig_edges_count) {
  
  # 1st filtering step: remove low expressed genes in each category
  # (genes under the percentile threshold)
  cut_off <- stats::quantile(as.matrix(assayData), prob = percentile,
                             na.rm = TRUE)
  user_message <- paste0("No gene above the threshold (", percentile * 100,
                         "th percentile).")
  
  category_median_list <- lapply(category_median_list, function(category_vec) {
    category_vec <- category_vec[category_vec > cut_off]
    if (length(category_vec) == 0) (
      category_vec <- user_message
    )
    return(category_vec)
  })
  
  # format to string vectors to allow easy detection of overlapping edges
  networks_to_string <- lapply(category_median_list, function(category_vec) {
    if (!is.character(category_vec)) {
      categoryEdges <- edges[edges$from %in% names(category_vec) & 
                               edges$to %in% names(category_vec), ]
      network_to_string <- do.call(paste, categoryEdges)
    } else {
      # assign error message for empty vectors
      network_to_string <- user_message
    }
    return(network_to_string)
  })
  
  # find overlapping edges across categories
  common_links <- lapply(contrasts, function(contrast) {
    return(intersect(
      networks_to_string[[contrast[1]]],
      networks_to_string[[contrast[2]]]
    ))
  }) %>%
    unlist() %>%
    unique()
  
  # remove error message from final list links 
  if (user_message %in% common_links) (
    common_links <- common_links[!(common_links %in% user_message)]
  )
  
  # 2nd filtering step
  # remove overlapping edges and format back to data.frame
  categories_network_list <- lapply(networks_to_string, function(subnetwork) {
    subnetwork <- subnetwork[!subnetwork %in% common_links]
    if (!(user_message %in% subnetwork)) (
      subnetwork <- data.frame(
        'from' = unlist(lapply(strsplit(subnetwork, " "), `[[`, 1)),
        'to' = unlist(lapply(strsplit(subnetwork, " "), `[[`, 2))
      )
    )
    return(subnetwork)
  })
  
  # count tot edges left per category
  tot_edges <- lapply(categories_network_list, function(category_network) {
    if (!is.character(category_network)) (
      nrow(category_network)
    )
  }) %>% 
    unlist() %>% 
    sum()
  
  # calculate interaction p values
  pvalues_list <- lapply(categories_network_list, function(category_network) {
    return(calc_pvalues_network(
      category_network = category_network,
      assayData = assayData,
      metadata = metadata,
      regression_method = regression_method,
      categories_length = categories_length,
      padj_method = padj_method
    ))
  })
  # count significant p values
  sig_var <- ifelse(padj_method == "none", "p.value", "p.adj")
  if (tot_edges > sig_edges_count) {
    # num tot edges greater than previous sig edges count
    p_values_sig_count <- lapply(pvalues_list, function(category_network) {
      if (!is.null(dim(category_network))) {
        num_sig_links <- sum(category_network[, sig_var] < 0.05, na.rm = TRUE)
        # return the number of links with significant p values or adj p
        if (!is.na(num_sig_links)) {
          return(num_sig_links)
        } else {return(0)}
      } else {return(0)}
    }) %>% unlist()
    num_sig_pvalues <- sum(p_values_sig_count) # these are p values or adj p
    
    # The <<- operator here is used only for efficiency reasons and does not
    # alter the .GlobalEnv as the variable sig_edges_count is defined in the 
    # parent environment of the get_diffNetworks_singleOmic() parent function 
    # (see
    # https://contributor.r-project.org/cran-cookbook/code_issues.html#writing-to-the-.globalenv).  
    # <<- represents the only way to implement early stopping of the loop 
    # when the total number of edges left in the network for a given 
    # percentile is smaller that the maximum number of significant edges found 
    # in previous percentile's networks. It wouldn't make sense to keep looking 
    # for a bigger number of significant edges in subsequent percentiles. 
    # The update of sig_edges_count will allow skipping unnecessary lm or rlm
    # iterations. 
    if (num_sig_pvalues > sig_edges_count) {
      sig_edges_count <<- num_sig_pvalues
    }
    pvalues_list <- append(pvalues_list, num_sig_pvalues)
    names(pvalues_list)[length(pvalues_list)] <- "sig_pvalues_count"
    return(pvalues_list)
  } else {
    # previous sig edges count greater than num tot edges, skip all calculations
    return(NULL)
  }
}


#' Calculate the p values for specific category network samples
#'
#' @inheritParams calc_pvalues_percentile
#' @param category_network network table for a specific category
#' @importFrom methods is
#' @importFrom MASS rlm
#' @importFrom sfsmisc f.robftest  
#' @return a list of p values
calc_pvalues_network <- function(assayData,
                                 metadata,
                                 padj_method,
                                 categories_length,
                                 regression_method = 'lm',
                                 category_network) {
  
  if (!is.character(category_network)) {
    if (nrow(category_network) > 0) {
      p.value <- vapply(seq_len(nrow(category_network)), function(i){
        gene_A <- category_network[i, 1]
        gene_B <- category_network[i, 2]
        
        if (categories_length == 2) {
          if (regression_method == "lm") {
            binary.metadata <- as.numeric(metadata) - 1
            design.mat <- cbind(
              1,
              as.numeric(assayData[gene_A, ]),
              binary.metadata,
              as.numeric(assayData[gene_A, ]) * binary.metadata
            )
            lmfit <- stats::.lm.fit(x = design.mat,
                                    y = as.numeric(assayData[gene_B, ]))
            dof <- length(lmfit$residuals) - length(lmfit$coefficients)
            # Mean Squared Error
            sigma_squared <- sum((lmfit$residuals)^2) / dof
            # variance-covariance of coefficients
            XtX_inv <- solve(t(design.mat) %*% design.mat)
            var_covar_matrix <- sigma_squared * XtX_inv
            se_coefficients <- sqrt(diag(var_covar_matrix))
            # t-statics for the interaction term (4th coeff) 
            t_stat <- (lmfit$coefficients[4]) / se_coefficients[4]
            p_interaction <- 2 * (1 - stats::pt(abs(t_stat), df = dof))
          }
          if (regression_method == "rlm") {
            # gene_B ~ gene_A * category
            robustfit <- rlm(as.numeric(assayData[gene_B, ]) ~
                    as.numeric(assayData[gene_A, ]) * metadata)

            p_interaction <- try(
              f.robftest(robustfit, var = 3)$p.value, silent = TRUE
            )
            if (inherits(p_interaction, "try-error")) (
              p_interaction <- NA_real_
            )
          }
        }
        if (categories_length >= 3) {
          # one-way ANOVA
          # gene_B ~ gene_A * category
          res_aov <- stats::aov(as.numeric(assayData[gene_B, ]) ~
                                  as.numeric(assayData[gene_A, ]) * metadata)
          p_interaction <- summary(res_aov)[[1]][["Pr(>F)"]][3]
        }
        return(p_interaction)
      }, FUN.VALUE = numeric(1))
    } else {
      p.value <- "No specific links for this category."
    }
  } else {
    p.value <- "No specific links for this category."
  }
  
  if (is.numeric(p.value)) {
    category_network <- cbind(category_network, p.value)
    
    if (padj_method == "q.value") {
      # adding Storey's q values
      p.adj <- try(qvalue::qvalue(category_network[, "p.value"])$qvalues,
                   silent = TRUE)
      if (is(p.adj, "try-error")) {
        if (nrow(category_network) > 1) (
          p.adj <- qvalue::qvalue(p = category_network[, "p.value"],
                                  pi0 = 1)$qvalues
        ) else (
          p.adj <- NA
        )
      }
      category_network$p.adj <- p.adj
    }
    
    if (!(padj_method %in% c("q.value", "none"))) {
      # adjust p values
      p.adj <- try(stats::p.adjust(category_network[, "p.value"],
                                   method = padj_method))
      if (is(p.adj, "try-error")) {
        p.adj <- NA
      }
      category_network$p.adj <- p.adj
    }
    
    rownames(category_network) <- paste(category_network$from,
                                        category_network$to, sep = "-")
  } else {
    category_network <- p.value
  }
  return(category_network)
}

#' Get a table of all the significant interactions across categories
#'
#' @param deggs_object an object of class `deggs` generated by
#' `get_diffNetworks`
#' @param assayDataName name of the assayData of interest. If an unnamed list of 
#' data was given to `get_diffNetworks`, `assayDataName` here will be the 
#' number corresponding to the position of the data in the `assayDataList`
#' provided before (i.e. if transcriptomic data was second in the list, a list 
#' of all its differential interactions can be obtained with assayDataName = 2, 
#' if only one data table was provided assayDataName must be 1). Default 1.
#' @param sig_threshold threshold for significance. Default 0.05.
#' @return a `data.frame` listing all the significant differential interactions
#' found across categories for that particular omic data.
#' This list can also be used to substitute or integrate feature selection in 
#' machine learning models for the prediction of the categories (see vignette).
#' @examples
#' data("synthetic_metadata")
#' data("synthetic_rnaseqData")
#' deggs_object <- get_diffNetworks(assayData = synthetic_rnaseqData,
#'                                  metadata = synthetic_metadata,
#'                                  category_variable = "response",
#'                                  verbose = FALSE,
#'                                  show_progressBar = FALSE,
#'                                  cores = 2)
#' get_sig_deggs(deggs_object, sig_threshold = 0.05) 
#' @export
get_sig_deggs <- function(deggs_object, 
                          assayDataName = 1, 
                          sig_threshold = 0.05) {
  
  if (!inherits(deggs_object, "deggs")) (
    stop("Input must be a 'deggs' object")
  )
  
  sig_var <- ifelse(deggs_object[["padj_method"]] == "none", "p.value", "p.adj")
  
  # Extract diffNetworks (=those that are lists) and exlude sig_pvalues_count
  is_list <- vapply(deggs_object[["diffNetworks"]][[assayDataName]],
                    is.list, logical(1))
  diffNetworks_list <- deggs_object[["diffNetworks"]][[assayDataName]][is_list]
  
  if (length(diffNetworks_list) == 0) {
    warning("No valid network found")
    return(data.frame())
  }
  
  # Extract all significant pairs across all categories
  sig_edges_list <- lapply(diffNetworks_list, function(subnetwork) {
    # Check if subnetwork has any rows
    if (is.null(subnetwork) || nrow(subnetwork) == 0) {
      return(data.frame())
    }
    
    # Filter significant edges
    sig_rows <- subnetwork[, sig_var] < sig_threshold
    if (any(sig_rows, na.rm = TRUE)) {
      return(subnetwork[sig_rows, , drop = FALSE])
    } else {
      return(data.frame())
    }
  })
  
  # Remove NULL entries and combine results
  sig_edges_list <- sig_edges_list[!vapply(sig_edges_list, is.null, logical(1))]
  
  if (length(sig_edges_list) == 0) {
    warning("No significant interactions found with ", 
            sig_var, " < ", sig_threshold)
    return(data.frame())
  }
  
  # Combine all significant edges
  sig_edges <- do.call(rbind, sig_edges_list)
  return(sig_edges)
}


#' Get a table of all significant interactions across categories
#'
#' @param deggs_object an object of class `deggs` generated by
#' `get_diffNetworks`
#' @param sig_threshold threshold for significance. Default 0.05.
#' @return a list of multilayer networks (as edge tables), one per 
#' category. 
#' @examples 
#' data("synthetic_metadata")
#' data("synthetic_rnaseqData")
#' data("synthetic_proteomicData")
#' data("synthetic_OlinkData")
#' assayData_list <- list("RNAseq" = synthetic_rnaseqData,
#'                        "Proteomics" = synthetic_proteomicData,
#'                        "Olink" = synthetic_OlinkData)
#' deggs_object <- get_diffNetworks(assayData = assayData_list,
#'                                  metadata = synthetic_metadata,
#'                                  category_variable = "response",
#'                                  verbose = FALSE,
#'                                  show_progressBar = FALSE,
#'                                  cores = 2)
#' get_multiOmics_diffNetworks(deggs_object, sig_threshold = 0.05)
#' @export
get_multiOmics_diffNetworks <- function(deggs_object,
                                        sig_threshold = 0.05) {
  
  if (!inherits(deggs_object, "deggs")) (
    stop("Input must be a 'deggs' object")
  )
  
  if (length(deggs_object[["assayData"]]) == 1) (
    stop("Only one omic dataset was provided. This function is used only in 
         multi-omic scenarios. Use get_sig_deggs() instead.")
  )
  
  categories <- levels(deggs_object[["metadata"]])
  omicDatasets <- names(deggs_object[["assayData"]])
  sig_var <- ifelse(deggs_object[["padj_method"]] == "none", "p.value", "p.adj")
  
  multiLayer_networks <- lapply(categories, function(category) {
    # Collect networks for this category from all omic datasets
    category_networks <- lapply(omicDatasets, function(omicDataset) {
      # Check if the omicDataset entry exists and is a list
      if (!is.list(deggs_object[["diffNetworks"]][[omicDataset]])) {
        return(data.frame())
      }
      
      # Extract the network for this specific category and omicDataset
      network <- deggs_object[["diffNetworks"]][[omicDataset]][[category]]
      
      # Check if the network is a valid data frame
      if (!is.data.frame(network)) {
        return(data.frame())
      }
      
      # Add a source column to identify the omicDataset and filter by sig
      network$layer <- omicDataset
      network <- network[which(network[, sig_var] < sig_threshold), ]
      
      return(network)
    })
    
    # Remove any NULL entries 
    category_networks <- category_networks[!vapply(category_networks, is.null,
                                                   logical(1))]
    
    # Combine all networks for this category
    if (length(category_networks) > 0) {
      merged_network <- do.call(rbind, category_networks)
      merged_network$layer <- as.factor(merged_network$layer)
      return(merged_network)
    } else {
      message("No valid category_networks found for category: ", category, ".")
      return(data.frame())
    }
  })
  names(multiLayer_networks) <- categories
  return(multiLayer_networks)
}

# get_diffNetworks - test -------------------------------------------------
test_that("test get_diffNetworks multi_omics", {
  # let's prepare a multi-omic example
  data("synthetic_metadata")
  data("synthetic_rnaseqData")
  data("synthetic_proteomicData")
  data("synthetic_OlinkData")
  
  assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                         "Proteomics" = synthetic_proteomicData,
                         "Olink" = synthetic_OlinkData)
  
  deggs_object <- get_diffNetworks(assayData = assayData_list,
                                   metadata = synthetic_metadata,
                                   category_variable = "response",
                                   regression_method = "lm",
                                   padj_method = "bonferroni",
                                   verbose = FALSE,
                                   show_progressBar = FALSE,
                                   cores = 1)
  
  # check the structure of the result is correct 
  expect_s3_class(deggs_object, "deggs")
  expect_named(deggs_object, c("diffNetworks", "assayData", "metadata", 
                               "regression_method", "category_subset",  
                               "padj_method"))
  expect_equal(deggs_object$assayData, assayData_list)
  expect_equal(length(deggs_object$assayData), length(assayData_list))
  
  expect_s3_class(deggs_object$metadata, "factor")
  expect_type(deggs_object$regression_method, "character")
  expect_type(deggs_object$padj_method, "character")
  
  # check the content hasn't changed 
  expect_equal(deggs_object[["diffNetworks"]][["RNAseq"]][["sig_pvalues_count"]],
               7)
  expect_equal(nrow(deggs_object[["diffNetworks"]][["RNAseq"]][["Responder"]]),
               3)
  expect_equal(deggs_object[["diffNetworks"]][["Proteomics"]][["Responder"]],
               "No specific links for this category.")
  expect_equal(ncol(deggs_object[["diffNetworks"]][["Olink"]][["Non_responder"]]),
               4)
  expect_equal(deggs_object[["diffNetworks"]][["Olink"]][["Responder"]],
               "No specific links for this category.")
  
  # even more granular check on the differential links found 
  expect_true("TNF" %in% 
        deggs_object[["diffNetworks"]][["RNAseq"]][["Non_responder"]][["from"]])
  expect_true("TNFRSF1A" %in% 
         deggs_object[["diffNetworks"]][["RNAseq"]][["Non_responder"]][["to"]])
})


# get_multiOmics_diffNetworks - test -------------------------------------------------
test_that("test get_multiOmics_diffNetworks multi_omics", {
# let's prepare a multi- omic example
  data("synthetic_metadata")
  data("synthetic_rnaseqData")
  data("synthetic_OlinkData")
  
  assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                         "Olink" = synthetic_OlinkData)
  
  deggs_object <- get_diffNetworks(assayData = assayData_list,
                                   metadata = synthetic_metadata,
                                   category_variable = "response",
                                   regression_method = "rlm",
                                   padj_method = "BH",
                                   verbose = FALSE,
                                   show_progressBar = FALSE,
                                   cores = 1)
res <- get_multiOmics_diffNetworks(deggs_object)

# check the structure of the result is correct 
expect_type(res, "list")

# check the content hasn't changed 
expect_equal(length(res), 2)
expect_equal(ncol(res$`Non_responder`), 5)
expect_equal(colnames(res$Responder), c("from", "to", "p.value", "p.adj",
                                        "layer"))
expect_equal(nrow(res$Responder), 3)

# even more granular check on the differential links found 
expect_true("AKT2" %in% res$`Non_responder`$from)
expect_true("MTOR" %in% res$`Non_responder`$to)

expect_true("FANCD2" %in% res$Responder$from)
expect_true("FAN1" %in% res$Responder$to)

})


# get_diffNetworks - test -------------------------------------------------
test_that("test get_diffNetworks single_omics", {
  # let's prepare a single omic example
  data("synthetic_metadata")
  data("synthetic_rnaseqData")
  
  deggs_object <- get_diffNetworks(assayData = synthetic_rnaseqData,
                                   metadata = synthetic_metadata,
                                   category_variable = "response",
                                   regression_method = "lm",
                                   padj_method = "bonferroni",
                                   verbose = FALSE,
                                   show_progressBar = FALSE,
                                   cores = 1)
  res <- get_sig_deggs(deggs_object)
  
  # check the structure of the result is correct 
  expect_s3_class(res, "data.frame")
  
  # check the content hasn't changed 
  expect_equal(ncol(res), 4)
  expect_equal(nrow(res), 7)
  
  # even more granular check on the differential links found 
  expect_true("TNF" %in% res$from)
  expect_true("TNFRSF1A" %in% res$to)
  
  expect_true("TGFB3" %in% res$from)
  expect_true("TGFBR1" %in% res$to)
  
  expect_true("IL1B" %in% res$from)
  expect_true("IL1R2" %in% res$to)
  
})

---
title: "Differential Network Analysis with multiDEGGs"
author: "Elisabetta Sciacca, Myles Lewis"
output:
  html_document:
    toc: true
    toc_float:
      collapsed: false
    toc_depth: 2
    number_sections: false
vignette: >
  %\VignetteIndexEntry{1. Differential Network Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction
The multiDEGGs package performs multi-omic differential network analysis by 
identifying differential interactions between molecular entities (genes, 
proteins, miRNAs, or other biomolecules) across the omic datasets provided.

For each omic dataset, a differential network is constructed, where links 
represent statistically significant differential interactions between entities. 
These networks are then integrated into a comprehensive visualization using 
distinct colors to distinguish interactions from different omic layers. This 
unified visualization allows interactive exploration of cross-omic patterns 
(e.g., differential interactions present at both transcript and protein level). 
For each link, users can access differential statistical significance metrics 
(p-values or adjusted p-values, calculated via robust or traditional linear 
regression with interaction term), and differential regression plots.

Beyond network visualization and exploration, multiDEGGs extends its utility 
into predictive modeling applications. The identified differential interactions 
can be leveraged as engineered features in machine learning pipelines, providing
biologically meaningful predictors that capture relational information 
between molecular entities. The package includes specialized functions for 
nested cross-validation that ensure proper feature selection and engineering 
without data leakage, enabling the construction of robust and interpretable 
predictive models (see `vignette("2. Feature Selection", package = "multiDEGGs")`
for details). 

## Installation
Install from CRAN:    
`install.packages("multiDEGGs")`
  
Install from Github:  
`devtools::install_github("elisabettasciacca/multiDEGGs")`

## Quick start - Generate Differential Networks

If you are working with human data, you can start differential analysis by 
using the internal multiDEGGs' default reference network.  
Users working with other species or requiring custom interaction sets can provide their own biological reference netowork as detailed in the last paragraph of this vignette.  
  
Let's start by loading the package and sample data:

```{r load_data}
library(multiDEGGs)
data("synthetic_metadata")
data("synthetic_rnaseqData")
data("synthetic_proteomicData")
data("synthetic_OlinkData")
```

Generate Differential Networks:

```{r}
assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                       "Proteomics" = synthetic_proteomicData,
                       "Olink" = synthetic_OlinkData)

deggs_object <- get_diffNetworks(assayData = assayData_list,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 regression_method = "lm",
                                 padj_method = "bonferroni",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 2)
```

### Key Parameters of `get_diffNetworks`

It's worth explaining some of the important parameters of `get_diffNetworks`:

* `assayData`: accepts either a single normalized matrix/data frame (for single 
  omic differential analysis), or a list of matrices/data frames (for multi-omic 
  scenarios). For multi-omic analysis, it's highly recommended to use a named 
  list of data. If unnamed, sequential names (assayData1, assayData2, etc.) will 
  be assigned to identify each matrix or data frame.

* `metadata`: can also be a named factor vector, with names matching the patient 
  IDs in column names of the assay data matrices/data frames. In that case, the 
  category_variable can remain unset (NULL by default).

* `category_subset`: this parameter can restrict the analysis to a certain 
  subset of categories available in the metadata/category vector.

* `regression_method`: set to `"lm"` by default because it is faster and highly 
  recommended in machine learning scenarios, where the function might be 
  repeatedly called many times. For basic differential analyses, `"rlm"` can 
  also be used and may perform better in some cases.

* `percentile_vector`: by default, molecular targets (genes, proteins, etc.) 
  whose expression level is below the 35th percentile of the entire data matrix 
  are excluded from the analysis. This threshold can be modified by specifying 
  the percentile vector that is internally used for the percolation analysis. 
  For example, to remove only targets below the 25th percentile, set 
  `percentile_vector = seq(0.25, 0.98, by = 0.05)`.

* `padj_method`: the default method is Bonferroni. Storey's q values often give 
  more generous results but the `qvalue` package needs to be installed first.

**NOTE**: Not all patient IDs need to be present across datasets. Different 
numbers of samples per omic are acceptable. Only IDs whose data is available in 
the colnames of the assayData will be included in the analysis. Missing IDs will 
be listed in a message similar to:

`The following samples IDs are missing in Proteomics: PT001, PT005, PT0030`

## Visualization

The `deggs_object` now contains the differential networks for each omic data
in `assayData_list`. These networks can be integrated into a comprehensive 
visualization where different colors distinguish links from different omic 
layers.

```{r, eval=FALSE}
View_diffNetworks(deggs_object)
``` 

This visualization interface allows to:

1. Navigate the networks associated with each patient category
2. Filter by link significance
3. Search for specific genes inside the network

<p>&nbsp;</p>
![](multiDEGGs_1.png){width=75%}
<p>&nbsp;</p> 

Thicker links correspond to higher significant p-values.  
The direction of the 
arrows shows the relationship direction reported in literature, not derived from 
the data.

The user can visualize differential regression plots by clicking on a link:

<p>&nbsp;</p>
![](multiDEGGs_2.png){width=75%}
<p>&nbsp;</p> 
<br> 
This **visual inspection of individual links is essential** before drawing any 
biological conclusions. These regression plots allow assessment of:

- **Effect size**: the magnitude of the difference in interaction strength 
between groups
- **Model fit quality**: how well the regression lines represent the data
- **Noise levels**: the scatter and confidence intervals around the regression lines
- **Outliers**: presence of influential data points that might drive the 
observed difference

<br> 
Single node differential expressions can also be visualized by clicking on the 
nodes:

<p>&nbsp;</p>
![](multiDEGGs_3.png){width=80%}
<p>&nbsp;</p> 

**NOTE**: For multi-omic scenarios, the data from the first matrix in the list 
passed to `assayData` will be used for this boxplot.

## List All Differential Interactions

Outside of the interactive environment, the `get_multiOmics_diffNetworks()` 
function can be used to get a table of all differential interactions, ordered by 
p-value or adjusted p-value:

```{r, warning=FALSE}
get_multiOmics_diffNetworks(deggs_object, sig_threshold = 0.05)
```
  
   
<p>For single omic scenarios, use the `get_sig_deggs()` function:</p>

```{r}
deggs_object_oneOmic <- get_diffNetworks(assayData = synthetic_rnaseqData,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 regression_method = "lm",
                                 padj_method = "bonferroni",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 2)

get_sig_deggs(deggs_object_oneOmic, sig_threshold = 0.05)
```

## Differential Regression Plots 

To plot the differential regression fits outside of the interactive environment, 
use `plot_regressions()` specifying the omic data to be used and the two targets:
  
```{r, fig.width = 4.5, fig.height = 4, eval=FALSE}
plot_regressions(deggs_object,
                 assayDataName = "RNAseq",
                 gene_A = "MTOR", 
                 gene_B = "AKT2",
                 legend_position = "bottomright")
```
  
<p>&nbsp;</p>
![](plot_regressions.png){width=50%}
  
In single omic analyses, the `assayDataName` parameter can remain unset.

## Differential Network Analysis with More Than Two Groups 

It's possible to compare differential interactions among more than two 
categorical groups. All steps described above stay the same;  
the dropdown 
menu of the interactive environment will show all available categories:

<table>
  <tr>
    <td><img src="multiDEGGs_4.png" width="70%"/></td>
  </tr>
</table>
  
<p> </p>

<p>While regressions and boxplots will show all categories: </p> 
  
   
<table>
  <tr>
    <td><img src="multiDEGGs_5.png" width="100%"/></td>
    <td><img src="multiDEGGs_6.png" width="100%"/></td>
  </tr>
</table>
  
The statistical significance of the interaction term is calculated via one-way 
ANOVA in this case.  
We highly recommend to have at least 4 or 5 observations per group.

## Custom Reference Network
The multiDEGGs package uses a human-specific reference network by default, 
consisting of 53,035 molecular interactions obtained both from `Omnipath` 
(Turei et al., Molecular Systems Biology, 2021) and the `MITHrIL` tool 
(Alaimo et al., Oncotarget, 2016).  
  
Users working with non-human species or requiring custom interaction sets can provide their own reference network using the `network` parameter of the `get_diffNetworks()` function.  
<br>

**Network Requirements**  
The custom network must be a `data.frame` with exactly two columns:

 - `from`: Origin node (e.g., Transcription Factor, miRNA, or Gene)
 - `to`: Target node
  
Nodes should use the same nomenclature (e.g., Gene Symbols, Entrez IDs) as the expression assay data to ensure proper mapping during the differential network analysis.  
<br>

**Sources for Other Species**  
If you are working with non-human species, several repositories allow for the extraction of interaction data:

 - **STRING-db**: Ideal for protein-protein interactions (PPI) across thousands of species. You can download the "protein.links" files
 - **KEGG** (via `KEGGgraph` or `TIGER`): Useful for metabolic and signaling pathways
 - **Omnipath**: A comprehensive resource that integrates multiple databases for human and mouse, easily accessible via the `OmnipathR` package
 - **miRBase** or **TargetScan**: For those needing specific miRNA-target interactions for non-human models  
 
<br>  
  
**Example: Generating a Custom Network in R**  
Below is a practical example of how to prepare a custom network, assuming you have a list of interactions:

```
# Example: Creating a toy network for a non-human species
# In a real scenario, you might load a CSV or use a database-specific package
interaction_data <- data.frame(
  source = c("GeneA", "GeneB", "miRNA1"),
  target = c("GeneC", "GeneA", "GeneB"),
  confidence = c(0.9, 0.8, 0.95)
)

# Formatting for get_diffNetworks() 
# We only need the 'from' and 'to' columns
custom_network <- interaction_data[, c("source", "target")]
colnames(custom_network) <- c("from", "to")

# Check class and structure
print(class(custom_network))
head(custom_network)

# Now this can be passed to the function:
diff_net <- get_diffNetworks(expr_data, network = custom_network, ...)
```
<br> 
  
**NOTE**  
Finally, it is important point to keep in mind that the differential networks obtained will be fundamentally limited by the reference biological network used. multiDEGGs can only identify interactions that are already present in the reference network, so using an incomplete or biased network might cause missing relevant biological relationships or, conversely, finding spurious associations. When choosing a reference network, it is important to assess its source, how comprehensive it is, and the quality of evidence supporting the interactions. These aspects should be documented when reporting multiDEGGs results.


## Citation
```{r}
citation("multiDEGGs")
```

---
title: "Feature Selection and Feature Engineering with multiDEGGs"
author: "Elisabetta Sciacca, Myles Lewis"
output:
  html_document:
    toc: true
    toc_float:
      collapsed: false
    toc_depth: 2
    number_sections: false
vignette: >
  %\VignetteIndexEntry{2. Feature Selection}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Feature Selection and Feature Engineering with multiDEGGs in Nested Cross-Validation

In computational biology applications involving high-throughput data, 
researchers commonly encounter situations where the number of potential 
predictors far exceeds the available sample size. This dimensional challenge 
requires careful feature selection strategies for both mathematical and clinical
reasons.
  
Standard feature selection methods typically evaluate predictors individually, 
identifying those variables that show the strongest univariate associations with
the outcome variable (such as through t-tests or Wilcoxon tests). 
While effective, this approach overlooks the interconnected nature of biological
systems, where \bold{informative patterns may emerge from relationships between 
variables rather than from individual measurements alone.}
  
Feature engineering represents a complementary strategy that creates new 
predictors by combining or transforming existing variables. In biology, 
such approach can be used to capture higher-order information that reflects the 
interconnected nature of molecular processes. For instance, the ratio between 
two genes may provide more discriminative power than either gene expression 
level independently, particularly when their relative balance is disrupted 
in disease states.
  
The informative content encoded in differential interactions, combined with 
multiDEGGs' ability to identify only literature-validated differential 
relationships, makes it particularly well-suited for both individual feature 
selection and guided creation of engineered predictors in machine learning. 
Such approach has potential to overcome the limitations of conventional 
algorithms which may select individual predictors without clear biological 
significance, compromising both the interpretability and clinical credibility 
of the resulting models. 
  
### Why Nested Cross-Validation for Feature Engineering?

It is crucial that feature selection and modification is conducted exclusively 
on training data within cross-validation loops to prevent information leakage 
from the test set. The `nestedcv` package enables the nested modification of 
predictors within each outer fold, ensuring that the attributes learned from 
the training part are applied to the test data without prior knowledge of the 
test data itself.  
The selected and combined features, and corresponding model, can then be 
evaluated on the hold-out test data without introducing bias.

Both `nestcv.glmnet` and `nestcv.train` from 
`nestedcv` accept any user-defined function 
that filters or transforms the feature matrix by passing the function name to 
the `modifyX` parameter.  
**The multiDEGGs package provides two specialized functions for this purpose.**

### multiDEGGs_filter(): Pure Differential Network-Based Selection

The `multiDEGGs_filter()` function performs feature selection based entirely on 
differential network analysis. It identifies significant differential molecular 
interactions and can return either the interaction pairs alone or both pairs 
and individual variables involved in those interactions.

#### Key Parameters

When using `multiDEGGs_filter()`, you can control the following parameters 
through `modifyX_options`:

- **`keep_single_genes`** (logical, default `FALSE`): Controls whether to 
include individual genes from significant pairs in addition to the pairs 
themselves
- **`nfilter`** (integer, default `NULL`): Maximum number of predictors to 
return. When `NULL`, all significant interactions found are included

#### Usage Examples

##### Basic Usage: Pairs Only

```{r, eval=FALSE}
library(multiDEGGs)
library(nestedcv)
data("synthetic_metadata")
data("synthetic_rnaseqData")

# Regularized linear model with interaction pairs only
fit.glmnet <- nestcv.glmnet(
  y = as.numeric(synthetic_metadata$response),
  x = t(synthetic_rnaseqData),
  modifyX = "multiDEGGs_filter",
  modifyX_options = list(
    keep_single_genes = FALSE,
    nfilter = 20
  ),
  modifyX_useY = TRUE,
  n_outer_folds = 5,
  n_inner_folds = 6,
  verbose = FALSE
)

summary(fit.glmnet)
```

```{r, echo=FALSE}
cat("Nested cross-validation with glmnet 
No filter 
Modifier:  multiDEGGs_filter 
Outer loop:  5-fold CV 
Inner loop:  6-fold CV 
100 observations, 14 predictors 
 
 
Final parameters: 
 lambda    alpha   
0.05894  0.10000   

Final coefficients: 
 (Intercept) TNF:TNFRSF1A    AKT2:MTOR   IL1B:IL1R2    FASLG:FAS TGFB3:TGFBR1  
    1.823874    -0.193020    -0.119887     0.052089    -0.035947    -0.033527  
MAP2K2:MAPK3  FANCD2:FAN1  
   -0.021308    -0.008862  
 
Result: 
       RMSE     R.squared   Pearson.r^2           MAE    
    0.47302       0.08148       0.09173       0.44153   " )
```
  
  
##### Including Individual Genes (keep_single_genes = TRUE)

```{r, eval=FALSE}
# Random forest model including both pairs and individual genes
fit.rf <- nestcv.train(
  y = synthetic_metadata$response,
  x = t(synthetic_rnaseqData),
  method = "rf",
  modifyX = "multiDEGGs_filter",
  modifyX_options = list(
    keep_single_genes = TRUE,
    nfilter = 30
  ),
  modifyX_useY = TRUE,
  n_outer_folds = 5,
  n_inner_folds = 6,
  verbose = FALSE
)

fit.rf$summary

# Plot ROC on outer folds
plot(fit.rf$roc)
```

```{r, echo=FALSE}
cat("               Reference
Predicted       Non_responder Responder
  Non_responder            57         2
  Responder                 1        40

              AUC            Accuracy   Balanced accuracy   
           0.9979              0.9700              0.9676   " )
```

![](roc.png){width=40%}
<p>&nbsp;</p>

#### How nfilter works with keep_single_genes

- When **`keep_single_genes = FALSE`**: `nfilter` limits only the number of 
interaction pairs returned
- When **`keep_single_genes = TRUE`**: `nfilter` limits the combined count 
of unique individual genes plus interaction pairs. The function prioritizes 
pairs by significance and adds individual genes as needed until the limit is 
reached

### multiDEGGs_combined_filter(): Hybrid Statistical and Network-Based Selection

The `multiDEGGs_combined_filter()` function combines traditional statistical 
feature selection with differential network analysis. This hybrid approach 
allows you to benefit from both conventional univariate selection methods and 
the biological insights from interaction analysis.

#### Key Parameters

- **`filter_method`** (character): Statistical method for single feature 
selection.  
Options: `"ttest"`, `"wilcoxon"`, `"ranger"`, `"glmnet"`, `"pls"`
- **`nfilter`** (integer): Maximum number of features to select
- **`dynamic_nfilter`** (logical): Controls how `nfilter` is applied 
(see detailed explanation below)
- **`keep_single_genes`** (logical): When `dynamic_nfilter = TRUE`, 
determines whether to include individual genes from multiDEGGs pairs

#### Dynamic vs. Balanced Selection Modes

##### Dynamic Selection (`dynamic_nfilter = TRUE`)

In dynamic mode, the function:
1. Selects `nfilter` single genes using the chosen statistical method
2. Adds ALL significant interaction pairs found by multiDEGGs
3. Total predictors = `nfilter` single genes + number of significant pairs

This mode allows the feature space to expand based on the biological complexity 
discovered in each fold.

```{r, eval=FALSE}
# Dynamic selection with t-test for single genes
fit.dynamic <- nestcv.glmnet(
  y = as.numeric(synthetic_metadata$response),
  x = t(synthetic_rnaseqData),
  modifyX = "multiDEGGs_combined_filter",
  modifyX_options = list(
    filter_method = "ttest", 
    nfilter = 20,
    dynamic_nfilter = TRUE, 
    keep_single_genes = FALSE
  ),
  modifyX_useY = TRUE,
  n_outer_folds = 5,
  n_inner_folds = 6,
  verbose = FALSE
)
```

##### Balanced Selection (`dynamic_nfilter = FALSE`)

In balanced mode, the function:  
1. Allocates approximately half of `nfilter` to interaction pairs  
2. Fills remaining slots with single genes from the statistical filter  
3. Maintains consistent total number of predictors across all folds  

This mode ensures a fixed feature space size while balancing single genes and 
interactions.

```{r, eval=FALSE}
# Balanced selection with Wilcoxon-test importance
fit.balanced <- nestcv.train(
  y = synthetic_metadata$response,
  x = t(synthetic_rnaseqData),
  method = "rf",
  modifyX = "multiDEGGs_combined_filter",
  modifyX_options = list(
    filter_method = "wilcoxon", 
    nfilter = 40,
    dynamic_nfilter = FALSE
  ),
  modifyX_useY = TRUE,
  n_outer_folds = 5,
  n_inner_folds = 6,
  verbose = FALSE
)
```
  
#### Available Statistical Methods

- **`"ttest"`**: Two-sample t-test for differential expression
- **`"wilcoxon"`**: Wilcoxon rank-sum test (non-parametric alternative to t-test)
- **`"ranger"`**: Random Forest variable importance scoring (the `ranger` package
must be installed first)
- **`"glmnet"`**: Elastic net regularization coefficients
- **`"pls"`**: Partial Least Squares variable importance

### Practical considerations

Before implementing multiDEGGs in your machine learning pipeline, it's highly 
recommended to first run a preliminary analysis on your complete dataset to 
assess the number of differential interactions detected. This exploratory step 
can guide your choice of approach and parameter settings.

If multiDEGGs identifies only a small number of differential interactions 
(e.g., fewer than 10-20 pairs), these features alone may lack sufficient 
predictive power. In such cases, consider:

- Using `multiDEGGs_combined_filter()` to integrate network-based features with 
traditional statistical selection methods
- Setting `keep_single_genes = TRUE` in `multiDEGGs_filter()` to include 
individual genes involved in the differential pairs
- Adjusting the `percentile_vector` or significance thresholds in the initial 
multiDEGGs analysis to potentially capture more interactions

Conversely, if a large number of differential interactions are detected, 
`multiDEGGs_filter()` alone may provide sufficient feature diversity for 
effective model training.

### Feature Engineering Details

Both functions create ratio-based features from significant gene pairs 
(Gene A / Gene B), which capture the relative expression relationships that 
drive differential network connectivity. The `predict` methods automatically 
handle the feature transformation for both training and test data within each 
cross-validation fold, ensuring no information leakage.

**Note:** If no significant differential interactions are found in a particular 
fold, both functions automatically fall back to t-test-based selection to ensure 
robust performance across all scenarios. This fallback is indicated by a 
printed "0" during execution.


## Citation
```{r}
citation("multiDEGGs")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_functions.R
\name{plot_regressions}
\alias{plot_regressions}
\title{Plot differential regressions for a link}
\usage{
plot_regressions(
  deggs_object,
  assayDataName = 1,
  gene_A,
  gene_B,
  title = NULL,
  legend_position = "topright"
)
}
\arguments{
\item{deggs_object}{an object of class \code{deggs} generated by
\code{get_diffNetworks}}

\item{assayDataName}{name of the assayData of interest. If an unnamed list of
data was given to \code{get_diffNetworks}, the assayDataName here will be the
number indicating the position of the data in the assayDataList provided
before (i.e. if the user wants to plot a differential interaction observed in
the transcriptomic data, which was second in the list, then assayDataName
must be 2, if only one data table was provided assayDataName must be 1).
Default 1.}

\item{gene_A}{character. Name of the first target (gene, protein, metabolite,
etc.)}

\item{gene_B}{character. Name of the second target (gene, protein, metabolite,
etc.)}

\item{title}{plot title. If NULL (default), the name of the assayData will be
used. Use empty character "" for no title.}

\item{legend_position}{position of the legend in the plot. It can be
specified by keyword or in any parameter accepted by \code{xy.coords} (defalut
"topright")}
}
\value{
base graphics plot showing differential regressions across
categories. The p value of the interaction term of
gene A ~ gene B \* category is reported on top.
}
\description{
Plot differential regressions for any target-target pair in an omic dataset
}
\examples{
data("synthetic_metadata")
data("synthetic_rnaseqData")
data("synthetic_proteomicData")
data("synthetic_OlinkData")
assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                       "Proteomics" = synthetic_proteomicData,
                       "Olink" = synthetic_OlinkData)
deggs_object <- get_diffNetworks(assayData = assayData_list,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 regression_method = "lm",
                                 padj_method = "bonferroni",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 1)
plot_regressions(deggs_object,
                 assayDataName = "RNAseq",
                 gene_A = "MTOR", 
                 gene_B = "AKT2",
                 legend_position = "bottomright")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_selection_for_ML.R
\name{cat_parallel}
\alias{cat_parallel}
\title{cat_parallel (from nestedcv)}
\usage{
cat_parallel(...)
}
\arguments{
\item{\dots}{to be passed to system()}
}
\description{
Prints using shell echo from inside mclapply when run in Rstudio
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_functions.R
\name{get_diffNetworks}
\alias{get_diffNetworks}
\title{Generate multi-omic differential networks}
\usage{
get_diffNetworks(
  assayData,
  metadata,
  category_variable = NULL,
  regression_method = "lm",
  category_subset = NULL,
  network = NULL,
  percentile_vector = seq(0.35, 0.98, by = 0.05),
  padj_method = "bonferroni",
  show_progressBar = TRUE,
  verbose = TRUE,
  cores = parallel::detectCores()/2
)
}
\arguments{
\item{assayData}{a matrix or data.frame (or list of matrices or data.frames
for multi-omic analysis) containing normalised assay data.
Sample IDs must be in columns and probe IDs (genes, proteins...) in rows.
For multi omic analysis, it is highly recommended to use a named list of
data.
If unnamed, sequential names (assayData1, assayData2, etc.) will be
assigned to identify each matrix or data.frame.}

\item{metadata}{a named vector, matrix, or data.frame containing sample
annotations or categories. If matrix or data.frame, each row should
correspond to a sample, with columns representing different sample
characteristics (e.g., treatment group, condition, time point). The colname
of the sample characteristic to be used for differential analysis must be
specified in \code{category_variable}. Rownames must match the sample IDs used in
assayData.
If named vector, each element must correspond to a sample characteristic to
be used for differential analysis, and names must match sample IDs used in
the colnames of \code{assayData}.
Continuous variables are not allowed.}

\item{category_variable}{when metadata is a matrix or data.frame this is the
column name of \code{metadata} that contains the sample annotations to be used for
differential analysis}

\item{regression_method}{whether to use robust linear modelling to calculate
link p values. Options are 'lm' (default) or 'rlm'. The lm implementation
is faster and lighter.}

\item{category_subset}{optional character vector indicating a subset of
categories from the category variable. If not specified, all categories in
\code{category_variable} will be used.}

\item{network}{network of biological interactions provided by the user. The
network must be provided in the form of a table of class data.frame with only
two columns named "from" and "to".
If NULL (default) a network of 10,537 molecular interactions obtained from
KEGG, mirTARbase, miRecords and transmiR will be used.
This has been obtained via the \code{exportgraph} function of the MITHrIL tool
(Alaimo et al., 2016).}

\item{percentile_vector}{a numeric vector specifying the percentiles to be
used in the percolation analysis. By default, it is defined as
\code{seq(0.35, 0.98, by = 0.05)}, which generates a sequence of percentiles
starting at 0.35, meaning that targets (genes/proteins...) whose expression
value is under the 35th percentile of the whole matrix will be excluded.
This threshold can be modified by specifying a different starting point for
\code{seq}. For a more granular percolation analysis an higher optimisation of
the algorithm, \code{by = 0.05} can be modified in favour of lower values, but
this will increase the computational time.}

\item{padj_method}{a character string indicating the p values correction
method for multiple test adjustment. It can be either one of the methods
provided by the \code{p.adjust} function from \code{stats} (bonferroni, BH, hochberg,
etc.) or "q.value" for Storey's q values, or "none" for unadjusted p values.
When using "q.value" the \code{qvalue} package must be installed first.}

\item{show_progressBar}{logical. Whether to display a progress bar during
execution. Default is TRUE.}

\item{verbose}{logical. Whether to print detailed output messages during
processing. Default is TRUE}

\item{cores}{number of cores to use for parallelisation.}
}
\value{
a \code{deggs} object containing differential networks incorporating
p values or adjusted p values for each link.
}
\description{
Generate a multi-layer differential network with interaction p values
}
\examples{
# Single omic analysis:
data("synthetic_metadata")
data("synthetic_rnaseqData")
deggs_object <- get_diffNetworks(assayData = synthetic_rnaseqData,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 regression_method = "lm",
                                 padj_method = "bonferroni",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 1)
 
# multi-omic analysis: 
data("synthetic_metadata")
data("synthetic_rnaseqData")
data("synthetic_proteomicData")
data("synthetic_OlinkData")
assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                       "Proteomics" = synthetic_proteomicData,
                       "Olink" = synthetic_OlinkData)
deggs_object <- get_diffNetworks(assayData = assayData_list,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 regression_method = "lm",
                                 padj_method = "bonferroni",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 1)
                                 
# to use only certain categories for comparison: 
# let's randomly add another level of response to the example metadata
synthetic_metadata$response <- as.character(synthetic_metadata$response)
indices <- sample(1:nrow(synthetic_metadata), 20, replace = FALSE) 
synthetic_metadata$response[indices] <- "Moderate response"
deggs_object <- get_diffNetworks(assayData = assayData_list,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 category_subset = c("Responder", 
                                                     "Non_responder"),
                                 regression_method = "lm",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 1)
                                 
# to be more generous on the targets to be excluded, and lower the expression 
# level threshold to the 25th percentile (or lower): 
deggs_object <- get_diffNetworks(assayData = assayData_list,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 category_subset = c("Responder", 
                                                     "Non_responder"),
                                 regression_method = "lm",
                                 percentile_vector = seq(0.25, 0.98, by = 0.05),
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 1)                                  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_functions.R
\name{View_diffNetworks}
\alias{View_diffNetworks}
\title{Interactive visualisation of differential networks}
\usage{
View_diffNetworks(deggs_object, legend.arrow.width = 0.35, stepY_legend = 55)
}
\arguments{
\item{deggs_object}{an object of class \code{deggs} generated by
\code{get_diffNetworks}}

\item{legend.arrow.width}{width of the arrow used in the network legend.
Default is 0.35. As the number of assayData matrices increases this parameter
must be accordingly increased to avoid graphical errors in the legend.}

\item{stepY_legend}{vertical space between legend arrows. It is used together
with \code{legend.arrow.width} to adjust the legend space in case of graphical
errors. Default is 55.}
}
\value{
a shiny interface showing networks with selectable nodes and links
}
\description{
Explore differential networks and interactively select regression
and box plots
}
\examples{
data("synthetic_metadata")
data("synthetic_rnaseqData")
data("synthetic_proteomicData")
data("synthetic_OlinkData")
assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                       "Proteomics" = synthetic_proteomicData,
                       "Olink" = synthetic_OlinkData)
deggs_object <- get_diffNetworks(assayData = assayData_list,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 regression_method = "lm",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 1)
# the below function runs a shiny app, so can't be run during R CMD check                                  
if(interactive()){
View_diffNetworks(deggs_object)  
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_functions.R
\name{calc_pvalues_percentile}
\alias{calc_pvalues_percentile}
\title{Compute interaction p values for a single percentile value}
\usage{
calc_pvalues_percentile(
  assayData,
  metadata,
  categories_length,
  category_median_list,
  padj_method,
  percentile,
  contrasts,
  regression_method,
  edges,
  sig_edges_count
)
}
\arguments{
\item{assayData}{a matrix or data.frame (or list of matrices or data.frames
for multi-omic analysis) containing normalised assay data.
Sample IDs must be in columns and probe IDs (genes, proteins...) in rows.
For multi omic analysis, it is highly recommended to use a named list of
data.
If unnamed, sequential names (assayData1, assayData2, etc.) will be
assigned to identify each matrix or data.frame.}

\item{metadata}{a named vector, matrix, or data.frame containing sample
annotations or categories. If matrix or data.frame, each row should
correspond to a sample, with columns representing different sample
characteristics (e.g., treatment group, condition, time point). The colname
of the sample characteristic to be used for differential analysis must be
specified in \code{category_variable}. Rownames must match the sample IDs used in
assayData.
If named vector, each element must correspond to a sample characteristic to
be used for differential analysis, and names must match sample IDs used in
the colnames of \code{assayData}.
Continuous variables are not allowed.}

\item{categories_length}{integer number indicating the number of categories}

\item{category_median_list}{list of category data.frames}

\item{padj_method}{a character string indicating the p values correction
method for multiple test adjustment. It can be either one of the methods
provided by the \code{p.adjust} function from \code{stats} (bonferroni, BH, hochberg,
etc.) or "q.value" for Storey's q values, or "none" for unadjusted p values.
When using "q.value" the \code{qvalue} package must be installed first.}

\item{percentile}{a float number indicating the percentile to use.}

\item{contrasts}{data.frame containing the categories contrasts in rows}

\item{regression_method}{whether to use robust linear modelling to calculate
link p values. Options are 'lm' (default) or 'rlm'. The lm implementation
is faster and lighter.}

\item{edges}{network of biological interactions in the form of a table of
class data.frame with two columns: "from" and "to".}

\item{sig_edges_count}{number of significant edges (p < 0.05)}
}
\value{
The list of float numbers of the significant pvalues for a single
percentile
}
\description{
Compute interaction p values for a single percentile value
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_data.R
\docType{data}
\name{synthetic_rnaseqData}
\alias{synthetic_rnaseqData}
\title{Synthetic RNA-seq count data}
\format{
A data frame with xx rows (genes) xx columns (patients IDs, matching
the metadata rownames).
}
\description{
Synthetic RNA-seq data after log2 normalisation
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_functions.R
\name{tidy_metadata}
\alias{tidy_metadata}
\title{Tidying up of metadata. Samples belonging to undesidered categories
(if specified) will be removed as well as categories with less than five
samples, and NAs.}
\usage{
tidy_metadata(
  category_subset = NULL,
  metadata,
  category_variable = NULL,
  verbose = FALSE
)
}
\arguments{
\item{category_subset}{optional character vector indicating which categories
are used for comparison. If not specified, all categories in
\code{category_variable} will be used.}

\item{metadata}{a named vector, matrix, or data.frame containing sample
annotations or categories. If matrix or data.frame, each row should
correspond to a sample, with columns representing different sample
characteristics (e.g., treatment group, condition, time point). The colname
of the sample characteristic to be used for differential analysis must be
specified in \code{category_variable}. Rownames must match the sample IDs used in
assayData.
If named vector, each element must correspond to a sample characteristic to
be used for differential analysis, and names must match sample IDs used in
the colnames of \code{assayData}.
Continuous variables are not allowed.}

\item{category_variable}{column name in \code{metadata} (if data.frame or matrix)
or NULL if \code{metadata} is already a named vector containing category
information.}

\item{verbose}{Logical. Whether to print detailed output messages
during processing. Default is FALSE.}
}
\value{
a tidy named factor vector of sample annotations.
}
\description{
Tidying up of metadata. Samples belonging to undesidered categories
(if specified) will be removed as well as categories with less than five
samples, and NAs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_selection_for_ML.R
\name{multiDEGGs_filter}
\alias{multiDEGGs_filter}
\title{multiDEGGs_filter}
\usage{
multiDEGGs_filter(y, x, keep_single_genes = FALSE, nfilter = NULL)
}
\arguments{
\item{y}{Numeric vector or factor. Response variable (outcome), i.e.
the 'metadata' named vector, as passed by \code{nestcv.train()} or
\code{nestcv.glmnet()} (package \pkg{nestedcv}).}

\item{x}{Predictor variables, i.e. the assayData matrix with genes in columns
and IDs in rows, as passed by \code{nestcv.train()} or
\code{nestcv.glmnet()} (package \pkg{nestedcv}).}

\item{keep_single_genes}{Logical, default FALSE. If TRUE, the function will
return unique individual genes along with significant pairs.}

\item{nfilter}{Integer. Maximum total number of predictors to return.
When \code{keep_single_genes = TRUE}, this parameter limits the combined count of
unique and paired predictors
(i.e., length(keep_DEGGs) + nrow(pairs) <= nfilter).
Predictors are included from most to least significant: for each pair, both
the pair itself and the new unique variables are included until the \code{nfilter}
threshold is reached.
When \code{keep_single_genes = FALSE}, \code{nfilter} only limits the number of pairs
returned.
If \code{NULL}, no filtering is applied and the total number of predictors
will depend on how many significantly different interactions are detected by
multiDEGGs in that fold.}
}
\value{
a list containing two types of predictors:
\itemize{
\item single predictors (stored in the 'keep_DEGGs' vector)
\item paired predictors (stored in the 'pairs' dataframe)
Note that \code{nfilter} limits the maximum number of engineered features returned,
however this number might be lower and will depend on how many significantly
different interactions will be found in each fold by multiDEGGs.
\bold{If no significantly different interactions are found
the function will print a '0' and switch to t-test for that fold.}
}
}
\description{
Function to be passed to the \code{modifyX} parameter of
\code{nestcv.train()} or \code{nestcv.glmnet()} (package \pkg{nestedcv}) to allow
nested feature selection and augmentation via differential network analysis
with multiDEGGs.
}
\examples{
\dontrun{
library(nestedcv)
data("synthetic_metadata")
data("synthetic_rnaseqData")

# fit a regularized linear model
# Note that nfilter, n_outer_folds, n_inner_folds are set low to keep the
# example lightweight. Adjust these values as needed for your use case.
fit.glmnet <- nestcv.glmnet(
  y = as.numeric(synthetic_metadata$response),
  x =  t(synthetic_rnaseqData),
  modifyX = "multiDEGGs_filter",
  modifyX_options = list(keep_single_genes = FALSE,
                         nfilter = 5),
  modifyX_useY = TRUE,
  n_outer_folds = 4,
  n_inner_folds = 4)

summary(fit.glmnet)

# fit a random forest model:
# note that nfilter, n_outer_folds, n_inner_folds are set low to keep the
# example lightweight. Adjust these values as needed for your use case.
fit.rf <- nestcv.train(
  y = synthetic_metadata$response,
  x = t(synthetic_rnaseqData),
  method = "rf",
  modifyX = "multiDEGGs_filter",
  modifyX_options = list(keep_single_genes = FALSE,
                         nfilter = 5),
  modifyX_useY = TRUE,
  n_outer_folds = 2,
  n_inner_folds = 2
)

fit.rf$summary
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_data.R
\docType{data}
\name{synthetic_metadata}
\alias{synthetic_metadata}
\title{Synthetic clinical data}
\format{
A data frame with 100 rows and 4 columns (IDs are in rownames):
\describe{
\item{patient_id}{IDs matching the IDs used in the colnames of the assay
data matrix/matrices.}
\item{age}{A column to simulate age of patients. Not used.}
\item{gender}{A column to simulate gender of patients. Not used.}
\item{response}{The response outcome, to be used for differential analysis}
}
}
\description{
A dataset containing sample clinical data for 100 patients with 40\%
response rate
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_selection_for_ML.R
\name{predict.multiDEGGs_filter}
\alias{predict.multiDEGGs_filter}
\title{Wrapper of .predict_multiDEGGs for multiDEGGs_filter()}
\usage{
\method{predict}{multiDEGGs_filter}(object, newdata, interaction.type = "ratio", sep = ":", ...)
}
\arguments{
\item{object}{A fitted object of class \code{multiDEGGs_filter} containing
filtering results with:
\describe{
\item{keep}{Character vector of variable names to keep as single
predictors}
\item{pairs}{Data frame or matrix with two columns specifying pairs
of variables to combine}
}}

\item{newdata}{A data frame containing the new data for prediction.
Must contain all variables specified in \code{object$keep} and
\code{object$pairs}.}

\item{interaction.type}{Character string specifying how to combine the
paired predictors.
Options are:
\describe{
\item{"ratio"}{Combine paired predictors by dividing the first variable
by the second (a/b)}
\item{other}{Combine paired predictors by multiplying the variables (a*b)
}
}
Default is "ratio".}

\item{sep}{Character string used as separator when creating column names for
combined predictors. Default is ":".}

\item{...}{Additional arguments passed to the generic function.}
}
\value{
A data frame containing:
\itemize{
\item Single predictors (if any are specified in \code{object$keep})
\item Combined predictors based on variable pairs and interaction type
}
}
\description{
This function generates predictions by creating a dataset with single
and combined predictors based on the filtering results of a
multiDEGGs_filter model.
}
\details{
The function processes the filtering results in two steps:
\enumerate{
\item Selects single predictors from \code{newdata} based on variables
listed in \code{object$keep}
\item Adds combined predictors from paired variables in
\code{object$pairs}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_functions.R
\name{get_sig_deggs}
\alias{get_sig_deggs}
\title{Get a table of all the significant interactions across categories}
\usage{
get_sig_deggs(deggs_object, assayDataName = 1, sig_threshold = 0.05)
}
\arguments{
\item{deggs_object}{an object of class \code{deggs} generated by
\code{get_diffNetworks}}

\item{assayDataName}{name of the assayData of interest. If an unnamed list of
data was given to \code{get_diffNetworks}, \code{assayDataName} here will be the
number corresponding to the position of the data in the \code{assayDataList}
provided before (i.e. if transcriptomic data was second in the list, a list
of all its differential interactions can be obtained with assayDataName = 2,
if only one data table was provided assayDataName must be 1). Default 1.}

\item{sig_threshold}{threshold for significance. Default 0.05.}
}
\value{
a \code{data.frame} listing all the significant differential interactions
found across categories for that particular omic data.
This list can also be used to substitute or integrate feature selection in
machine learning models for the prediction of the categories (see vignette).
}
\description{
Get a table of all the significant interactions across categories
}
\examples{
data("synthetic_metadata")
data("synthetic_rnaseqData")
deggs_object <- get_diffNetworks(assayData = synthetic_rnaseqData,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 2)
get_sig_deggs(deggs_object, sig_threshold = 0.05) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_functions.R
\name{get_multiOmics_diffNetworks}
\alias{get_multiOmics_diffNetworks}
\title{Get a table of all significant interactions across categories}
\usage{
get_multiOmics_diffNetworks(deggs_object, sig_threshold = 0.05)
}
\arguments{
\item{deggs_object}{an object of class \code{deggs} generated by
\code{get_diffNetworks}}

\item{sig_threshold}{threshold for significance. Default 0.05.}
}
\value{
a list of multilayer networks (as edge tables), one per
category.
}
\description{
Get a table of all significant interactions across categories
}
\examples{
data("synthetic_metadata")
data("synthetic_rnaseqData")
data("synthetic_proteomicData")
data("synthetic_OlinkData")
assayData_list <- list("RNAseq" = synthetic_rnaseqData,
                       "Proteomics" = synthetic_proteomicData,
                       "Olink" = synthetic_OlinkData)
deggs_object <- get_diffNetworks(assayData = assayData_list,
                                 metadata = synthetic_metadata,
                                 category_variable = "response",
                                 verbose = FALSE,
                                 show_progressBar = FALSE,
                                 cores = 2)
get_multiOmics_diffNetworks(deggs_object, sig_threshold = 0.05)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_functions.R
\name{node_boxplot}
\alias{node_boxplot}
\title{Boxplots of single nodes (genes,proteins, etc.)}
\usage{
node_boxplot(gene, assayDataName = 1, deggs_object)
}
\arguments{
\item{gene}{gene name  (must be in \code{rownames(assayData)})}

\item{assayDataName}{name of the assayData of interest. If an unnamed list of
data was given to \code{get_diffNetworks}, the assayDataName here will be the
number indicating the position of the data in the assayDataList provided
before (i.e. if the user wants to plot a differential interaction observed in
the transcriptomic data, which was second in the list, then assayDataName
must be 2, if only one data table was provided assayDataName must be 1).
Default 1.}

\item{deggs_object}{an object of class \code{deggs} generated by
\code{get_diffNetworks}}
}
\value{
the boxplot
}
\description{
This function is for internal use of \code{View_diffnetworks}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_functions.R
\name{my_palette}
\alias{my_palette}
\title{Internal function for colors}
\usage{
my_palette(n)
}
\arguments{
\item{n}{number of colors needed}
}
\value{
a vector with colors
}
\description{
This function return a color palette with the number of colors specified by n
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_selection_for_ML.R
\name{multiDEGGs_combined_filter}
\alias{multiDEGGs_combined_filter}
\title{Combined multiDEGGs filter}
\usage{
multiDEGGs_combined_filter(
  y,
  x,
  filter_method = "ttest",
  nfilter,
  dynamic_nfilter = TRUE,
  keep_single_genes = FALSE,
  ...
)
}
\arguments{
\item{y}{Numeric vector or factor. Response variable (outcome), i.e.
the 'metadata' named vector, as passed by \code{nestcv.train()} or
\code{nestcv.glmnet()} (package \pkg{nestedcv}).}

\item{x}{Predictor variables, i.e. the assayData matrix with genes in columns
and IDs in rows, as passed by \code{nestcv.train()} or
\code{nestcv.glmnet()} (package \pkg{nestedcv}).}

\item{filter_method}{Character string. Statistical filtering method to be
used in combination with multiDEGGs for sigle feature selection.
Options are: "ttest", "wilcoxon", "ranger", "glmnet", "pls".}

\item{nfilter}{Integer. Maximum number of features to select.}

\item{dynamic_nfilter}{Logical. If \code{TRUE} \code{nfilter} will limit the number of
features selected by the statistical filter and the feature space will be
augmented by adding ALL the paired predictors found by multiDEGGs.
If \code{FALSE} \code{nfilter} will limit the total number of predictors, with
approximately half allocated to pairs and half to single genes.}

\item{keep_single_genes}{Logical. When \code{dynamic_nfilter = TRUE}, determines
whether to include single genes selected by multiDEGGs (i.e. the single
variables included in the differential pairs) in addition to those from
the statistical filter. Default is FALSE.}

\item{...}{Additional arguments passed to the filtering functions.}
}
\value{
An object of class "multiDEGGs_filter" containing:
\item{keep}{Character vector of selected single gene names}
\item{pairs}{Data frame of selected gene pairs with interaction information}
}
\description{
This function can be passed to the \code{modifyX} parameter of
\code{nestcv.train()} or \code{nestcv.glmnet()} (package \pkg{nestedcv})
to use one of the available statistical filters (t-test, wilcoxon, etc.) in
combination with multiDEGGs.
Single predictors will be selected by the selected statistical filter an
paired predictors will be added by multiDEGGs.
}
\details{
The function operates in two modes:

\strong{Dynamic Filtering (dynamic_nfilter = TRUE):}
\itemize{
\item Selects \code{nfilter} single genes using the specified statistical method
\item Finds all significant gene pairs using multiDEGGs
\item Total predictors = nfilter single genes + number of significant pairs
\itemize{
\item If \code{keep_single_genes = TRUE}, also includes single genes obtained
from pairs found by multiDEGGs
}
}

\strong{Balanced Selection (dynamic_nfilter = FALSE):}
\itemize{
\item Allocates approximately half of \code{nfilter} to gene pairs
\item Remaining slots filled with single genes from the statistical filter
\item If fewer pairs are found than allocated, compensates by selecting more
single genes
\item Ensures consistent total number of predictors across outer folds
}

The statistical filtering methods include:
\itemize{
\item \code{"ttest"}: Two-sample t-test for differential expression
\item \code{"wilcoxon"}: Wilcoxon rank-sum test
\item \code{"ranger"}: Random Forest variable importance
\item \code{"glmnet"}: Elastic net regularization
\item \code{"pls"}: Partial Least Squares variable importance
}
}
\examples{
\dontrun{
library(nestedcv)
data("synthetic_metadata")
data("synthetic_rnaseqData")

# fit a regularized linear model
# note that nfilter, n_outer_folds, n_inner_folds are set low to keep the
# example lightweight. Adjust these values as needed for your use case.
fit.glmnet <- nestedcv::nestcv.glmnet(
y = as.numeric(synthetic_metadata$response),
x =  t(synthetic_rnaseqData),
modifyX = "multiDEGGs_combined_filter",
modifyX_options = list(filter_method = "ttest", 
                       nfilter = 5,
                       dynamic_nfilter = TRUE, 
                       keep_single_genes = FALSE),
modifyX_useY = TRUE,
n_outer_folds = 4,
n_inner_folds = 4)

summary(fit.glmnet)

# fit a random forest model
# NOTE: nfilter, n_outer_folds, n_inner_folds are set low to keep the
# example lightweight. Adjust these values as needed for your use case.
fit.rf <- nestedcv::nestcv.train(
  y = synthetic_metadata$response,
  x = t(synthetic_rnaseqData),
  method = "rf",
  modifyX = "multiDEGGs_combined_filter",
  modifyX_options = list(filter_method = "ttest", 
                         nfilter = 5,
                         dynamic_nfilter = TRUE, 
                         keep_single_genes = FALSE),
  modifyX_useY = TRUE,
  n_outer_folds = 2,
  n_inner_folds = 2
)

fit.rf$summary
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_functions.R
\name{get_diffNetworks_singleOmic}
\alias{get_diffNetworks_singleOmic}
\title{Generate differential networks for single omic analysis}
\usage{
get_diffNetworks_singleOmic(
  assayData,
  assayDataName,
  metadata,
  regression_method,
  network,
  percentile_vector,
  padj_method,
  show_progressBar,
  verbose,
  cores
)
}
\arguments{
\item{assayData}{a matrix or data.frame (or list of matrices or data.frames
for multi-omic analysis) containing normalised assay data.
Sample IDs must be in columns and probe IDs (genes, proteins...) in rows.
For multi omic analysis, it is highly recommended to use a named list of
data.
If unnamed, sequential names (assayData1, assayData2, etc.) will be
assigned to identify each matrix or data.frame.}

\item{assayDataName}{name of the assayData, to identify which omic is.}

\item{metadata}{a named vector, matrix, or data.frame containing sample
annotations or categories. If matrix or data.frame, each row should
correspond to a sample, with columns representing different sample
characteristics (e.g., treatment group, condition, time point). The colname
of the sample characteristic to be used for differential analysis must be
specified in \code{category_variable}. Rownames must match the sample IDs used in
assayData.
If named vector, each element must correspond to a sample characteristic to
be used for differential analysis, and names must match sample IDs used in
the colnames of \code{assayData}.
Continuous variables are not allowed.}

\item{regression_method}{whether to use robust linear modelling to calculate
link p values. Options are 'lm' (default) or 'rlm'. The lm implementation
is faster and lighter.}

\item{network}{network of biological interactions provided by the user. The
network must be provided in the form of a table of class data.frame with only
two columns named "from" and "to".
If NULL (default) a network of 10,537 molecular interactions obtained from
KEGG, mirTARbase, miRecords and transmiR will be used.
This has been obtained via the \code{exportgraph} function of the MITHrIL tool
(Alaimo et al., 2016).}

\item{percentile_vector}{a numeric vector specifying the percentiles to be
used in the percolation analysis. By default, it is defined as
\code{seq(0.35, 0.98, by = 0.05)}, which generates a sequence of percentiles
starting at 0.35, meaning that targets (genes/proteins...) whose expression
value is under the 35th percentile of the whole matrix will be excluded.
This threshold can be modified by specifying a different starting point for
\code{seq}. For a more granular percolation analysis an higher optimisation of
the algorithm, \code{by = 0.05} can be modified in favour of lower values, but
this will increase the computational time.}

\item{padj_method}{a character string indicating the p values correction
method for multiple test adjustment. It can be either one of the methods
provided by the \code{p.adjust} function from \code{stats} (bonferroni, BH, hochberg,
etc.) or "q.value" for Storey's q values, or "none" for unadjusted p values.
When using "q.value" the \code{qvalue} package must be installed first.}

\item{show_progressBar}{logical. Whether to display a progress bar during
execution. Default is TRUE.}

\item{verbose}{logical. Whether to print detailed output messages during
processing. Default is TRUE}

\item{cores}{number of cores to use for parallelisation.}
}
\value{
a list of differential networks, one per category
}
\description{
Generate differential networks for single omic analysis
}
